wide_window = true
wide_window = false
include("load_InSe.jl")

# Run WFPT assuming zero doping

# wfpt : If true, include rest-space self-energy. If false, do not.

for nk in [30, 60, 120, 180]

    postfix = "nk$nk"
    occ = setup_occupation_params(0.0)

    η_rest = 100.0 * unit_to_aru(:meV)
    nq_rest = 60
    window_wfpt = (-Inf, Inf) .* unit_to_aru(:eV)

    kpts, _, _, nstates_base = filter_kpoints((nk, nk, 1), model.nw, model.el_ham, window_active; model.symmetry)
    kpts = GridKpoints(kpts)
    kqpts, ik_to_ikirr_isym = ElectronPhonon.unfold_kpoints(kpts, model.symmetry)
    ikq_to_ik = getindex.(ik_to_ikirr_isym, 1)

    @info "kpts.n = $(kpts.n)"
    @info "kqpts.n = $(kqpts.n)"

    # Compute rest space contribution using WFPT
    Σs_rest = zeros(ComplexF64, kpts.n, model.nw, length(occ))
    @time res = compute_self_energy_rest_space(
        kpts, occ, model, folder, outdir, prefix;
        η_rest, window_active, window_wfpt, nqs_rest = (nq_rest, nq_rest, 1))

    Σs_rest_DW = zeros(ComplexF64, kpts.n, model.nw, length(occ))
    Σs_rest_Fan = zeros(ComplexF64, kpts.n, model.nw, length(occ))
    Σs_rest_rest_fan = zeros(ComplexF64, kpts.n, model.nw, length(occ))
    Σs_rest_rest_dw = zeros(ComplexF64, kpts.n, model.nw, length(occ))

    # Reshape into (ik, ib, iT) matrix
    for ik in 1:kpts.n
        Σs_rest[ik, :, :] .= real(res.Σ_rest[ik])
        Σs_rest_DW[ik, :, :] .= real(res.Σ_wfpt.Σ_DW[ik])
        Σs_rest_Fan[ik, :, :] .= real(res.Σ_wfpt.Σ_Fan[ik])
        Σs_rest_rest_fan[ik, :, :] .= real(res.Σ_rest_fan[ik])
        Σs_rest_rest_dw[ik, :, :] .= real(res.Σ_rest_dw[ik])
    end

    jldsave("$folder_output/data_wfpt.nk$nk.jld2"; Σs_rest, kpts, kqpts, ikq_to_ik, nq_rest, window_wfpt, η_rest, Σs_rest_DW, Σs_rest_Fan, Σs_rest_rest_fan, Σs_rest_rest_dw)

    # Compute and save BTStates
    el_states = compute_electron_states(model, kpts, ["eigenvector", "velocity"], window_active; fourier_mode = "gridopt")
    el_i, = ElectronPhonon.electron_states_to_BTStates(el_states, kpts, nstates_base)
    jldsave("$folder_output/data_scKubo_BTStates_nk$nk.jld2"; el_i)
end;
