wide_window = true
include("load_InSe.jl")

# for nk in [30, 60, 120, 180], wfpt in [false, true]
for nk in [30, 60, 120, 180], wfpt in [true]

    nq = nk

    ind_doping = 1
    n_doping = n_doping_list[ind_doping]
    occ = setup_occupation_params(n_doping)

    postfix = "n$n_doping.wfpt$wfpt.nk$nk"

    # Compute Fermi level
    if !ElectronPhonon.chemical_potential_is_computed(occ)
        kpts, _, _, nstates_base = filter_kpoints((nk, nk, 1), model.nw, model.el_ham, window_active; model.symmetry)
        el_states = compute_electron_states(model, kpts, ["eigenvalue", "velocity_diagonal"], window_active; fourier_mode = "gridopt")
        el, _ = ElectronPhonon.electron_states_to_BTStates(el_states, kpts, nstates_base)
        ElectronPhonon.bte_compute_μ!(occ, el; do_print=true)
    end

    η = 5.0 * unit_to_aru(:meV)

    kpts = ElectronPhonon.Kpoints(vcat(
        [Vec3(x, x, 0) for x in range(-div(nk, 3), 0, length=div(nk, 3)+1) ./ nk],
        [Vec3(x, 0, 0) for x in range(0, div(nk, 2), length=div(nk, 2)+1) ./ nk][2:end],
    ))
    kpts, = filter_kpoints(kpts, model.nw, model.el_ham, window_active)


    # Compute rest space contribution using WFPT
    Σs_rest = zeros(ComplexF64, kpts.n, model.nw, length(occ))

    if wfpt
        filename_wfpt = "$folder_output/data_wfpt.nk$nk.jld2"
        isfile(filename_wfpt) || error("WFPT data file not found : $filename_wfpt")
        Σs_rest_grid, kqpts_grid, ikq_to_ik_grid = load(filename_wfpt, "Σs_rest", "kqpts", "ikq_to_ik")

        for ik in 1:kpts.n
            ikq = xk_to_ik(kpts.vectors[ik], kqpts_grid)
            if ikq === nothing
                @warn "ikq not found for ik = $ik"
                continue
            end

            ik_grid = ikq_to_ik_grid[ikq]
            if ik == 0
                @warn "ik_grid not found for ik = $ik"
                continue
            end

            Σs_rest[ik, :, :] .= Σs_rest_grid[ik_grid, :, :]
        end
    end

    kqpts, = filter_kpoints((nq, nq, 1), model.nw, model.el_ham, window_active; model.symmetry)
    kqpts = GridKpoints(kqpts)
    kqpts, _ = ElectronPhonon.unfold_kpoints(kqpts, model.symmetry)

    @info "kpts.n = $(kpts.n)"
    @info "kqpts.n = $(kqpts.n)"

    calc = ElectronSpectralCalculatorX6X(; occ, ωlist = ωs, smearing = η)
    @time res = ElectronPhonon.run_eph_over_k_and_kq(model, kpts, kqpts, [calc]; window_kq = window_active, progress_print_step = 10)

    # Add rest contribution
    for ik in 1:kpts.n, ib in 1:model.nw, iT in 1:length(occ)
        calc.Σ[ik][:, ib, iT] .+= Σs_rest[ik, ib, iT]
    end

    es = hcat(getproperty.(res.el_k_save, :e)...)

    As_Dyson = zeros(length(ωs), model.nw, kpts.n, length(occ))
    for iT in 1:length(occ), ik in 1:kpts.n, ib in 1:model.nw
        ek = es[ib, ik]
        Σs = calc.Σ[ik][:, ib, iT]
        As_Dyson[:, ib, ik, iT] = @. -imag(1 / (ωs - ek - Σs)) / π
    end

    # =================================================================================
    # Run cumulant on xks_cum using interpolated self-energy

    Σs = calc.Σ
    ωs = calc.ωlist

    xplot = norm.(Ref(model.recip_lattice) .* kpts.vectors) .* sign.(getindex.(kpts.vectors, 1))
    Σs_all = cat(Σs..., dims=4);

    xks_cum = vcat(
        [Vec3(x, x, 0) for x in range(-1/3, 0., length=101)],
        [Vec3(x, 0, 0) for x in range(0., 0.5, length=101)][2:end],
    )
    xplot_cum = norm.(Ref(model.recip_lattice) .* xks_cum) .* sign.(getindex.(xks_cum, 1))
    es_cum = compute_eigenvalues_el(model, ElectronPhonon.Kpoints(xks_cum))

    ts = range(0, 2e4, step=5.)
    ωs_cum = EPSpectral.fftfreq_t2w(ts)
    inds_cum = searchsortedfirst(ωs_cum, ωs[1]-e_Fermi-0.1) : searchsortedlast(ωs_cum, ωs[end]-e_Fermi+0.1)

    @show diff(ωs_cum)[1] / unit_to_aru(:meV)
    @show ωs_cum[end] / unit_to_aru(:meV)

    basis = LinearSplineBasis(ωs .- e_Fermi)

    ibs_and_iks = collect(Iterators.product(1:model.nw, 1:length(xks_cum)))
    As_cum = zeros(length(inds_cum), model.nw, length(xks_cum), length(occ));
    es_onshell = zeros(ComplexF64, model.nw, length(xks_cum), length(occ));

    for iT in 1:length(occ)
        @time As_cum_ = tmapreduce(hcat, ibs_and_iks) do (ib, ik)
            if Threads.threadid() == 1
                @info "ib = $ib, ik = $ik"
            end
            if !(window_active[1] <= es_cum[ib, ik] <= window_active[2])
                # Skip if es[ib, ik] is outside the window
                return zeros(length(ωs_cum))
            else
                # Interpolate self-energy
                Σs_itp = extrapolate(interpolate!((ωs, xplot), view(Σs_all, :, ib, iT, :), Gridded(Linear())), Flat())
                Σs_rest_itp = linear_interpolation((xplot,), Σs_rest[:, ib, iT]; extrapolation_bc = Flat())

                Σs_k = Σs_itp.(ωs, xplot_cum[ik])
                Σ_static = Σs_rest_itp(xplot_cum[ik])

                # Add rest self-energy to ek not Σs (because it is purely static)
                ek = es_cum[ib, ik] + real(Σ_static)
                Σs_k .= Σs_k .- Σ_static

                βs = .-imag.(linear_interpolation(ωs, Σs_k; extrapolation_bc=0).(basis.xs[2:end-1] .+ ek)) ./ π
                return EPSpectral.run_cumulant(basis, βs, ek - e_Fermi, ts, ts)
            end
        end
        As_cum[:, :, :, iT] .= reshape(As_cum_, :, model.nw, length(xks_cum))[inds_cum, :, :]

        for ik in 1:length(xks_cum), ib in 1:model.nw
            ek = es_cum[ib, ik]
            if !(window_active[1] <= ek <= window_active[2])
                es_onshell[ib, ik, iT] = ek
            else
                Σs_itp = extrapolate(interpolate!((ωs, xplot), view(Σs_all, :, ib, iT, :), Gridded(Linear())), Flat())
                es_onshell[ib, ik, iT] = ek + Σs_itp(ek, xplot_cum[ik])
            end
        end
    end;

    jldsave("$folder_output/data_cumulant_1d.eta5meV.$postfix.jld2";
        xks_cum, es_cum, ts, basis, As_cum, es_onshell, occ, ωs_cum = ωs_cum[inds_cum] .+ e_Fermi);
end;
