wide_window = false
include("load_InSe.jl")


for longrange_mode in [:quadrupole, :none, :dipole]
    
    # Load model with a different long-range treatment

    if longrange_mode == :none
        folder = "/globalscratch/ucl/modl/jmlihm/eph_self_consistent/InSe/1_epw.nosoc"
    elseif longrange_mode == :dipole
        folder = "/globalscratch/ucl/modl/jmlihm/eph_self_consistent/InSe/1_epw.nosoc.polar"
    elseif longrange_mode == :quadrupole
        folder = "/globalscratch/ucl/modl/jmlihm/eph_self_consistent/InSe/1_epw.nosoc.quad"
    end

    model = ElectronPhonon.load_model_from_epw_new(folder, outdir, prefix);

    if longrange_mode == :none
        model.polar_phonon.dynmat_asr .*= 0
        model.polar_phonon.Z .*= 0
        model.polar_phonon.Q .*= 0
        model.polar_eph.Z .*= 0
        model.polar_eph.Q .*= 0
    end
    if longrange_mode == :none || longrange_mode == :dipole
        model.el_pos.op_r .= 0
    end

    mobility_bte_SI_dict = Dict()
    mobility_serta_SI_dict = Dict()
    γs_dict = Dict()
    es_dict = Dict()
    μlist_dict = Dict()

    filename_output = "$folder_output/data_bte.longrange.$longrange_mode.jld"
    if isfile(filename_output)
        mobility_bte_SI_dict = load(filename_output, "mobility_bte_SI_dict")
        mobility_serta_SI_dict = load(filename_output, "mobility_serta_SI_dict")
        γs_dict = load(filename_output, "γs_dict")
        es_dict = load(filename_output, "es_dict")
        μlist_dict = load(filename_output, "μlist_dict")
    end

    η = 5.0 * unit_to_aru(:meV)

    for nk in [30, 60, 120, 180], ind_doping in [6, 7]
        key = (nk, ind_doping)

        (; σ_bte_SI, σ_serta_SI, mobility_bte_SI, mobility_serta_SI, γs, es, occ) = run_BTE(nk, ind_doping; η, nk_tuple = (nk, nk, 1))

        mobility_serta_SI_dict[key] = mobility_serta_SI[1, 1, :]
        mobility_bte_SI_dict[key] = mobility_bte_SI[1, 1, :]
        γs_dict[key] = γs
        es_dict[key] = es
        μlist_dict[key] = occ.μlist
    end

    Tlist = setup_occupation_params(0.).Tlist
    jldsave("$folder_output/data_bte.longrange.$longrange_mode.jld"; mobility_bte_SI_dict, mobility_serta_SI_dict, γs_dict, es_dict, μlist_dict, Tlist)
end;
