wide_window = false
include("load_InSe.jl")


# Calculate doping dependence at fixed temperature

function setup_occupation_params(n_doping)
    occ = ElectronOccupationParams(;
        Tlist = 100.0 .* unit_to_aru(:K),
        nlist = -(10.0 .^ range(10, 14.5, length = 15)) .* (model.volume / model.lattice[3, 3] / unit_to_aru(:cm)^2),
        volume = model.volume,
        nelec = 2,
        spin_degeneracy = 2,
    )
    return occ
end

n_doping_list = [1.]  # Placeholder to indicate doping, value is not used.
folder_output = "$folder/../2_julia.prl.newwfpt/doping_dependence"

begin
    # Copy wfpt and BTStates data to doping_dependence folder
    for nk in [30, 60, 120, 180]
        res = load("$folder_output/../data_wfpt.nk$nk.jld2")

        Σs_rest = res["Σs_rest"]
        kpts = res["kpts"]
        kqpts = res["kqpts"]
        ikq_to_ik = res["ikq_to_ik"]

        occ = setup_occupation_params(0.)
        Σs_rest_new = zeros(size(Σs_rest)[1], size(Σs_rest)[2], length(occ))
        for i in 1:length(occ)
            Σs_rest_new[:, :, i] .= Σs_rest[:, :, 3]
        end

        jldsave("$folder_output/data_wfpt.nk$nk.jld2"; Σs_rest = Σs_rest_new, kpts, kqpts, ikq_to_ik)

        el_i = load("$folder_output/../data_scKubo_BTStates_nk$nk.jld2", "el_i")
        jldsave("$folder_output/data_scKubo_BTStates_nk$nk.jld2"; el_i)
    end
end;

begin
    # Run BTE
    ind_doping = 1

    for η_tag in ["smearing_5meV", "smearing_1meV"]

        mobility_bte_SI_dict = Dict()
        mobility_serta_SI_dict = Dict()
        μlist_dict = Dict()

        if η_tag === "smearing_5meV"
            filename_output = "$folder_output/data_bte.jld"
        else
            filename_output = "$folder_output/data_bte.$η_tag.jld"
        end

        if isfile(filename_output)
            mobility_bte_SI_dict = load(filename_output, "mobility_bte_SI_dict")
            mobility_serta_SI_dict = load(filename_output, "mobility_serta_SI_dict")
            μlist_dict = load(filename_output, "μlist_dict")
        end

        if η_tag === "smearing_5meV"
            η = 5.0 * unit_to_aru(:meV)
        elseif η_tag === "smearing_1meV"
            η = 1.0 * unit_to_aru(:meV)
        elseif η_tag === "smearing_T"
            η = occ.Tlist[1]
        else
            error("Unknown η_tag")
        end

        for nk in [30, 60, 120, 180]
            key = (nk, ind_doping)
            haskey(mobility_bte_SI_dict, key) && continue

            res = run_BTE(nk, ind_doping; η, nk_tuple = (nk, nk, 1));
            mobility_serta_SI_dict[key] = res.mobility_serta_SI[1, 1, :]
            mobility_bte_SI_dict[key] = res.mobility_bte_SI[1, 1, :]
            μlist_dict[key] = res.occ.μlist
        end

        nlist_SI = @. -setup_occupation_params(0.).nlist / (model.volume / model.lattice[3, 3] / unit_to_aru(:cm)^2)
        Tlist = setup_occupation_params(0.).Tlist

        jldsave(filename_output; mobility_bte_SI_dict, mobility_serta_SI_dict, μlist_dict, Tlist, nlist_SI, η)
    end
end

for nk in [30, 60, 120, 180]
    wfpt = true
    ind_doping = 1

    filename_BTStates = "$folder_output/data_scKubo_BTStates_nk$nk.jld2"
    el_i = load("$folder_output/data_scKubo_BTStates_nk$nk.jld2", "el_i")

    postfix = "doping$ind_doping.wfpt$wfpt.nk$nk"

    # ============================================================
    # Run scFM
    run_scFM(nk, wfpt, ind_doping; nk_tuple = (nk, nk, 1))

    # ============================================================
    for η_tag in ["smearing_5meV", "smearing_1meV", "smearing_T"]
    # for η_tag in ["smearing_5meV"]
        # ============================================================
        # Run Dyson with and without truncation
        occ = setup_occupation_params(0.)
        if η_tag === "smearing_5meV"
            η = 5.0 * unit_to_aru(:meV)
        elseif η_tag === "smearing_1meV"
            η = 1.0 * unit_to_aru(:meV)
        elseif η_tag === "smearing_T"
            η = occ.Tlist[1]
        else
            error("Unknown η_tag")
        end
        η_list = fill(η, length(occ.Tlist))

        run_Dyson(nk, wfpt, ind_doping; η_tag, η_list, nk_tuple = (nk, nk, 1))

        run_Dyson(nk, wfpt, ind_doping; η_tag = η_tag * ".no_truncation", η_list, nk_tuple = (nk, nk, 1), truncate_spectral = false)

        # ============================================================
        # Run cumulant

        occ = setup_occupation_params(0.)
        if η_tag === "smearing_5meV"
            η = 5.0 * unit_to_aru(:meV)
        elseif η_tag === "smearing_1meV"
            η = 1.0 * unit_to_aru(:meV)
        elseif η_tag === "smearing_T"
            η = occ.Tlist[1]
        else
            error("Unknown η_tag")
        end
        η_list = fill(η, length(occ.Tlist))

        μlist_prev = zeros(length(occ))
        μlist_new = zeros(length(occ))

        for iter in 1:20
            if iter == 1
                # Load Dyson data
                filename = "$folder_output/data_Dyson.$η_tag.no_truncation.$postfix.jld2"
                occ, kpts, ωs_dense, Σs_rest, Σs, ωs = load(filename, "occ", "kpts", "ωs_dense", "Σs_rest", "Σs", "ωs");

            else
                # Run Dyson (with updated chemical potential)
                η_tag_tmp = "temporary_Dyson_for_cumulant.$η_tag"
                run_Dyson(nk, wfpt, ind_doping; η_tag = η_tag_tmp, η_list, nk_tuple = (nk, nk, 1), truncate_spectral = false, update_μ = false, μlist_input = μlist_new)
                filename = "$folder_output/data_Dyson.$η_tag_tmp.$postfix.jld2"

                occ, kpts, ωs_dense, Σs_rest, Σs, ωs = load(filename, "occ", "kpts", "ωs_dense", "Σs_rest", "Σs", "ωs");
                rm(filename)  # Remove temporary file as it is no longer needed
            end

            # Run Cumulant
            Σs = scKubo_trim_Σs_to_BTStates(Σs, el_i, kpts);
            Σs_rest = scKubo_trim_Σs_rest_to_BTStates(Σs_rest, el_i, kpts);
            @time As_cum = run_cumulant_spectral(ωs, Σs, Σs_rest, el_i, occ, e_Fermi, ωs_dense);

            # Update chemical potential
            μlist_prev .= occ.μlist
            μlist_new .= cumulant_compute_chemical_potential(As_cum, occ, el_i, ωs_dense)

            @info "μlist_new = $(_e_to_eV.(μlist_new))"

            jldsave("$folder_output/data_cumulant.$η_tag.$postfix.jld2"; As_cum, occ, ωs_dense)

            if all(abs.(μlist_new .- μlist_prev) .< 0.01 * unit_to_aru(:meV))
                break
            end
        end
    end
end;


begin
    # Run bubble mobility calculations
    wfpt = true
    ind_doping = 1

    mob_SI_bubble_dict = Dict()
    μlist_dict = Dict()

    if isfile("$folder_output/data_bubble.jld")
        mob_SI_bubble_dict, μlist_dict = load("$folder_output/data_bubble.jld", "mob_SI_bubble_dict", "μlist_dict")
    end

    occ = setup_occupation_params(0.)
    Tlist_Kelvin = @. occ.Tlist / unit_to_aru(:K)
    nlist_SI = @. -occ.nlist / (model.volume / model.lattice[3, 3] / unit_to_aru(:cm)^2)

    # Cumulant mobility
    for nk in [30, 60, 120, 180], η_tag in ["smearing_$x" for x in ["5meV", "1meV", "T"]]
        prefix = "data_cumulant.$η_tag"
        postfix = "doping$ind_doping.wfpt$wfpt.nk$nk"

        key = (nk, wfpt, ind_doping, prefix)
        haskey(mob_SI_bubble_dict, key) && continue

        @info key

        filename = "$folder_output/$prefix.$postfix.jld2"
        isfile(filename) || continue
        As_cum, occ = load(filename, "As_cum", "occ");

        filename_BTStates = "$folder_output/data_scKubo_BTStates_nk$nk.jld2"
        el_i = load("$folder_output/data_scKubo_BTStates_nk$nk.jld2", "el_i");

        # i_vbm = argmax(el_i.e)
        # e_vbm_renorm = ωs_dense[argmax(As_cum[:, i_vbm, end])]

        (; nlist, σ) = cumulant_compute_conductivity(0., As_cum, occ, el_i, ωs_dense)

        fac_mobility_SI = ElectronPhonon.e2 / (unit_to_aru(:A) / unit_to_aru(:V) * unit_to_aru(:cm)^2) ./ ElectronPhonon.units.e_SI
        mobility = σ[1, 1, :] ./ (abs.(occ.nlist) / model.volume) .* fac_mobility_SI

        for iT in 1:length(occ)
            if abs((nlist[iT] - occ.nlist[iT]) / occ.nlist[iT]) > 1e-2
                println("WARNING: Occupation not converged for iT = $iT")
                println("T = $(occ.Tlist[iT] / unit_to_aru(:K)) K")
                println("n_target = $(occ.nlist[iT])")
                println("n_calc   = $(nlist[iT])")
                println("Setting mobility to NaN")
                mobility[iT] = NaN
            end
        end

        mob_SI_bubble_dict[key] = mobility
        μlist_dict[key] = occ.μlist

        jldsave("$folder_output/data_bubble.jld"; mob_SI_bubble_dict, μlist_dict, Tlist_Kelvin, nlist_SI)

    end

    prefix_list = vcat(vec(["data_Dyson.smearing_$x$y" for x in ["5meV", "1meV", "T"] for y in ["", ".no_truncation"]]), "data_scFM")

    for nk in [30, 60, 120, 180], prefix in prefix_list

        key = (nk, wfpt, ind_doping, prefix)
        haskey(mob_SI_bubble_dict, key) && continue

        @info key

        truncate_spectral = occursin("Dyson", prefix) && !occursin("no_truncation", prefix)
        @time res = run_bubble(nk, wfpt, ind_doping, prefix; truncate_spectral, phonon_assisted = false)

        if res !== nothing
            mob_SI_bubble_dict[key] = res.mobility[1, 1, :]
            μlist_dict[key] = res.occ.μlist
        end

        jldsave("$folder_output/data_bubble.jld"; mob_SI_bubble_dict, μlist_dict, Tlist_Kelvin, nlist_SI)

        flush(stdout)
        flush(stderr)
    end
end;
