using ElectronPhonon
using EPSpectral
using PyPlot
using LinearAlgebra
using Interpolations
using OhMyThreads
using JLD2
using ThreadPinning; pinthreads(:cores)
using EPSpectral
using EPSpectral: occ_boson, occ_fermion, docc_fermion, grid_points
using Roots
using JSON3

folder_code = "/home/ucl/modl/jmlihm/eph_self_consistent/InSe/MaterialsCloud/code/dev"
include("$folder_code/electron_spectral.jl")
include("$folder_code/electron_selfen.jl")
include("$folder_code/ab_initio_solver.jl")
include("$folder_code/green_conductivity.jl")
include("$folder_code/scKubo.jl")
include("$folder_code/scKubo_bubble.jl")
include("$folder_code/run_Dyson.jl")
include("$folder_code/run_scFM.jl")
include("$folder_code/run_bubble.jl")
include("$folder_code/run_cumulant.jl")
include("$folder_code/run_BTE.jl")

folder = "../2_epw"
folder = "/globalscratch/ucl/modl/jmlihm/eph_self_consistent/InSe/1_epw.nosoc.quad.newwfpt"
outdir = "./temp"
prefix = "InSe"

model = ElectronPhonon.load_model_from_epw_new(folder, outdir, prefix);

e_Fermi = -5.940 * unit_to_aru(:eV)
ω_LO = 26.7 * unit_to_aru(:meV)
_e_to_eV(e) = (e - e_Fermi) / unit_to_aru(:eV)

if wide_window
    folder_output = "./data/wide_window"
    window_active = (-0.7, 0.3) .* unit_to_aru(:eV) .+ e_Fermi  # 2_julia.prl/wide_window
    ωs = Vector(range(-0.8, 0.5, step=0.001) .* unit_to_aru(:eV) .+ e_Fermi)
    ωs_dense = range(-0.8, 0.5, step=0.001) .* unit_to_aru(:eV) .+ e_Fermi
else
    folder_output = "./data"
    window_active = (-0.5, 0.3) .* unit_to_aru(:eV) .+ e_Fermi  # 2_julia.prl
    ωs = Vector(range(-0.6, 0.5, step=0.001) .* unit_to_aru(:eV) .+ e_Fermi)
    ωs_dense = range(-0.6, 0.5, step=0.001) .* unit_to_aru(:eV) .+ e_Fermi
end

# @info 0.02892 / (model.volume / model.lattice[3, 3] / unit_to_aru(:cm)^2)
n_doping_list = [0.0, 0.00723, 0.01446, 0.02892, 0.001446, 0.00001446, 0.000001446]
# 0, 5e12 cm⁻², 1e13 cm⁻², 2e13 cm⁻², 1e12 cm⁻², 1e10 cm⁻², 1e9 cm⁻²


function setup_occupation_params(n_doping)
    occ = ElectronOccupationParams(;
        Tlist = [50.0, 75.0, 100.0, 150.0, 200.0, 250.0, 300.0] .* unit_to_aru(:K),
        nlist = -n_doping,
        volume = model.volume,
        nelec = 2,
        spin_degeneracy = 2,
    )
    if n_doping == 0
        occ.μlist .= +Inf
    end
    return occ
end
