function load_and_interpolate_spectral_function_3d_itp(filenames, ωs_plot, xks_plot, nk, nband; ib_rng = 1:nband, η_plot = nothing)
    filename = filenames[1]
    ωs = load(filename, "ωs")
    Σs = load(filename, "Σs")
    kqpts = load(filename, "kqpts")
    occ = load(filename, "occ")
    ikq_to_ik = load(filename, "ikq_to_ik")

    # Setup the unfolding map. Size nk+1 to include both 0 and 1.
    iks_to_ik_irr = zeros(Int, nk+1, nk+1, nk+1)
    for (ikq, xkq) in enumerate(kqpts.vectors)
        xkq = ElectronPhonon.normalize_kpoint_coordinate.(xkq)
        iks = round.(Int, xkq .* nk) .+ 1
        iks_to_ik_irr[iks...] = ikq_to_ik[ikq]
    end
    iks_to_ik_irr[nk+1, :, :] .= iks_to_ik_irr[1, :, :]
    iks_to_ik_irr[:, nk+1, :] .= iks_to_ik_irr[:, 1, :]
    iks_to_ik_irr[:, :, nk+1] .= iks_to_ik_irr[:, :, 1]

    Σs_all = [zeros(ComplexF64, length(ωs), nk+1, nk+1, nk+1) for _ in 1:nband]
    Σs_all_itp = [interpolate!((ωs, range(0, 1, length=nk+1), range(0, 1, length=nk+1), range(0, 1, length=nk+1)), Σs_all[ib], Gridded(Linear())) for ib in 1:nband]


    Σs_list = []
    As_list = []

    for filename in filenames

        Σs = load(filename, "Σs")

        Σs_plot = zeros(ComplexF64, length(ωs_plot), nband, length(xks_plot), length(occ.Tlist))
        As_plot = zeros(length(ωs_plot), nband, length(xks_plot), length(occ.Tlist))

        es_renorm = zeros(nband, length(xks_plot), length(occ.Tlist))

        @time for iT in 1:length(occ.Tlist)

            # Unfold self-energy to the full BZ, update Σs_all.
            for iks in CartesianIndices((nk+1, nk+1, nk+1))
                ik = iks_to_ik_irr[iks]
                ik == 0 && continue
                for ib in 1:nband
                    Σs_all[ib][:, iks] .= Σs[:, ib, ik, iT]
                end
            end

            for ib in ib_rng
                @views for ik in eachindex(xks_plot)
                    xk = ElectronPhonon.normalize_kpoint_coordinate(xks_plot[ik])
                    Σs_plot[:, ib, ik, iT] = Σs_all_itp[ib].(ωs_plot, xk...)

                    Σs_with_η = copy(Σs_plot[:, ib, ik, iT])

                    if η_plot !== nothing
                        inds = @. -imag(Σs_with_η) < η_plot
                        Σs_with_η[inds] .= real.(Σs_with_η[inds]) .- im .* η_plot
                    end

                    ek = es_plot[ib, ik]
                    As_plot[:, ib, ik, iT] .= @. -imag(1 / (ωs_plot - ek - Σs_with_η)) / π
                    es_renorm[ib, ik, iT] = ωs_plot[argmax(As_plot[:, ib, ik, iT])]
                end
            end
        end

        push!(Σs_list, Σs_plot)
        push!(As_list, As_plot)
    end

    (; As_list, Σs_list, occ)
end


function load_and_interpolate_spectral_function_1d_itp(filenames, ωs_plot, xks_plot, xplot, get_xks_on_grid, nband, es_plot; ib_rng = 1:nband, η_plot = 1e-8)

    Σs_list = []
    As_list = []

    for filename in filenames
        @time Σs = load(filename, "Σs")
        ωs = load(filename, "ωs")
        kqpts = load(filename, "kqpts")
        occ = load(filename, "occ")
        ikq_to_ik = load(filename, "ikq_to_ik")
        Σs_rest_offset = nothing
        try
            Σs_rest_offset = load(filename, "Σs_rest_offset")
        catch e
        end

        xks_on_grid, xplot_on_grid = get_xks_on_grid(kqpts.ngrid)

        Σs_sc = zeros(ComplexF64, length(ωs_plot), nband, length(xks_plot), length(occ.Tlist))
        As_sc = zeros(length(ωs_plot), nband, length(xks_plot), length(occ.Tlist))

        @time for iT in 1:length(occ.Tlist)

            Σs_itp_data = [zeros(ComplexF64, length(ωs), length(xks_on_grid)) for _ in 1:nband]
            for i in eachindex(xks_on_grid)
                xk = xks_on_grid[i]
                ikq = xk_to_ik(xk, kqpts)

                ikq === nothing && continue
                ik = ikq_to_ik[ikq]
                ik == 0 && continue
                for ib in 1:nband
                    Σs_itp_data[ib][:, i] .= Σs[:, ib, ik, iT]
                end
            end

            if Σs_rest_offset !== nothing
                ωs_shifted = ωs .- real(Σs_rest_offset[iT])
                ωs_plot_shifted = ωs_plot .- real(Σs_rest_offset[iT])
            else
                ωs_shifted = ωs
                ωs_plot_shifted = ωs_plot
            end
            Σs_itp = [extrapolate(interpolate!((ωs, xplot_on_grid), Σ, Gridded(Linear())), 0) for Σ in Σs_itp_data]

            for ib in ib_rng
                @views for ik in eachindex(xks_plot)
                    Σs_sc[:, ib, ik, iT] .= Σs_itp[ib].(ωs_plot_shifted, xplot[ik])

                    Σs_with_η = copy(Σs_sc[:, ib, ik, iT])
                    if η_plot !== nothing
                        inds = @. -imag(Σs_with_η) < η_plot
                        Σs_with_η[inds] .= real.(Σs_with_η[inds]) .- im .* η_plot
                    end

                    ek = es_plot[ib, ik]
                    As_sc[:, ib, ik, iT] .= @. -imag(1 / (ωs_plot_shifted - ek - Σs_with_η)) / π
                end
            end

            if Σs_rest_offset !== nothing
                Σs_sc[:, :, :, iT] .+= Σs_rest_offset[iT]
            end
        end

        push!(Σs_list, Σs_sc)
        push!(As_list, As_sc)
    end

    (; As_list, Σs_list, occ)
end


function load_and_interpolate_spectral_function_1d_itp_linewidth(filename, ωs_plot, xks_plot, xplot, xks_on_grid, xplot_on_grid, nband; ib_rng = 1:nband, η_plot = 1e-8, Σs_rest = nothing)
    ωs = load(filename, "ωs")
    kqpts = load(filename, "kqpts")
    kqpts = load(filename, "kqpts")
    occ = load(filename, "occ")
    ikq_to_ik = load(filename, "ikq_to_ik")
    εks = load(filename, "εks")

    @time Σs = load(filename, "Σs")

    Σ_shift = zeros(length(occ.Tlist))
    As_sc_noshift = zeros(length(ωs_plot), nband, length(xks_plot), length(occ.Tlist))
    As_sc_shift = zeros(length(ωs_plot), nband, length(xks_plot), length(occ.Tlist))

    if occursin("NaCl", filename)
        ib0, ik0 = argmin(εks).I
        iω0 = argmin(abs.(ωs .- εks[ib0, ik0]))
    else
        error("Unknown material")
    end

    @time for iT in 1:length(occ.Tlist)
        Σ_shift[iT] = real(Σs[iω0, ib0, ik0, iT])

        Σs_itp_data = [zeros(ComplexF64, length(ωs), length(xks_on_grid)) for _ in 1:nband]
        for i in eachindex(xks_on_grid)
            xk = xks_on_grid[i]
            ikq = xk_to_ik(xk, kqpts)

            ikq === nothing && continue
            ik = ikq_to_ik[ikq]
            ik == 0 && continue

            for ib in 1:nband
                Σs_itp_data[ib][:, i] .= Σs[:, ib, ik, iT]
            end

            if Σs_rest !== nothing
                for ib in 1:nband
                    Σs_itp_data[ib][:, i] .+= Σs_rest[ik, ib, iT]
                end
            end
        end

        Σs_itp = [extrapolate(interpolate!((ωs, xplot_on_grid), Σ, Gridded(Linear())), 0) for Σ in Σs_itp_data]

        for ib in ib_rng
            @views for ik in eachindex(xks_plot)
                Σs_with_η = Σs_itp[ib].(ωs_plot, xplot[ik])
                if η_plot !== nothing
                    inds = @. -imag(Σs_with_η) < η_plot
                    Σs_with_η[inds] .= real.(Σs_with_η[inds]) .- im .* η_plot
                end

                ek = es_plot[ib, ik]
                As_sc_noshift[:, ib, ik, iT] .= @. -imag(1 / (ωs_plot - ek - Σs_with_η)) / π
                As_sc_shift[:, ib, ik, iT] .= @. -imag(1 / (ωs_plot - ek - Σs_with_η + Σ_shift[iT])) / π
            end
        end
    end

    (; As_sc_noshift, As_sc_shift, Σ_shift, occ)
end




function _plot_spectral(plotaxes_A_2d, plotaxes_A_1d, data, iT, kwargs_A;
                        e_unit_type, c_onshell = "b",
                        As_labels = nothing,
                        As_list = nothing, 
                        plot_horizontal_lines = true,
                        ik0 = nothing,
                        plot_RSPT = true,)
    (; ωs_plot, xplot, es_plot, es_onshell, ω_LO, e_ref, ib_rng, Tlist) = data
    if ik0 === nothing
        ik0 = findfirst(xplot .≈ 0)
    end

    if e_unit_type == :eV
        e_unit = unit_to_aru(:eV)
    elseif e_unit_type == :ω₀
        e_unit = 1.0
    else
        error("Invalid e_unit_type $e_unit_type")
    end
    _e_to_eV(e) = (e - e_ref) / e_unit

    extent = [extrema(xplot)..., _e_to_eV.(extrema(ωs_plot))...]

    img = nothing
    for (i, key) in enumerate(["FM", "Cumulant", "scFM"])
        if As_list !== nothing
            if i > length(As_list)
                break
            end
            As = As_list[i]
            key = As_labels[i]
        else
            As = data.As_dict[key]
        end

        if plot_RSPT
            ax = plotaxes_A_2d[i+1]
        else
            ax = plotaxes_A_2d[i]
        end
        z = dropdims(sum(As[:, ib_rng, :, iT], dims=2), dims=2) .* e_unit

        inds = xplot .<= 0
        extent_ = [extent[1], 0, extent[3], extent[4]]
        img = ax.imshow(z[:, inds] ; extent = extent_, origin="lower", aspect="auto", kwargs_A...)

        inds = xplot .>= 0
        extent_ = [0, extent[2], extent[3], extent[4]]
        img = ax.imshow(z[:, inds]; extent = extent_, origin="lower", aspect="auto", kwargs_A...)

        # c = [color1, color2, color3][i]

        label = key
        key == "Cumulant" && (label = "Cum.")
        # key == "FM" && (label = "DM")
        ls = ["-.", "--", "-"][i]


        if plotaxes_A_1d !== nothing
            plotaxes_A_1d.plot(z[:, ik0], _e_to_eV.(ωs_plot); c, label, ls)
        end

        if key == "scFM" && plot_horizontal_lines
            e0 = ωs_plot[argmax(z[:, ik0])]

            axes_ = [ax]
            if plotaxes_A_1d !== nothing
                push!(axes_, plotaxes_A_1d)
            end

            if Tlist[iT] > 0
                for ax_ in axes_
                    ax_.axhline.(_e_to_eV.([0.] .+ e0); c = color3, lw=0.8, ls="-")
                    ax_.axhline.(_e_to_eV.([ω_LO, -ω_LO] .+ e0); c = color3, lw=0.8, ls="--")
                end
            else
                for ax_ in axes_
                    ax_.axhline.(_e_to_eV.([0.] .+ e0); c = color3, lw=0.8, ls="-")
                    ax_.axhline.(_e_to_eV.([ω_LO] .+ e0); c = color3, lw=0.8, ls="--")
                end
            end
        end
    end

    # Plot bare bands
    for ax in plotaxes_A_2d
        for ib in ib_rng
            ax.plot(xplot, _e_to_eV.(es_plot[ib, :]), "k--"; lw = 1)
        end
    end

    # Plot renormalized bands for on shell
    if es_onshell !== nothing && plot_RSPT
        for ib in ib_rng
            plotaxes_A_2d[1].plot(xplot, _e_to_eV.(real.(es_onshell[ib, :, iT])), "-"; lw = 1, c = c_onshell)
        end
    end

    return img
end


function plot_spectral(data, plotaxes_A_2d, plotaxes_A_1d;
        As_list = [data.As_dict["FM"], data.As_dict["Cumulant"], data.As_dict["scFM"]],
        cax, iT, vmax, vmin = vmax / 1e2, e_unit_type, ymin, ymax,
        logscale = true, cmap = "Blues",
        kwargs...,)

    if logscale
        kwargs_A = (; cmap, norm=matplotlib.colors.LogNorm(vmin, vmax))
    else
        kwargs_A = (; cmap, vmin, vmax = vmax)
    end
    img = _plot_spectral(plotaxes_A_2d, plotaxes_A_1d, data, iT, kwargs_A; As_list, e_unit_type, kwargs...)
    cbar = colorbar(img; cax)

    if e_unit_type == :eV
        label_A = "\$A_\\mathbf{k}\\ (1 / \\mathrm{eV})\$"
        label_e = "\$\\varepsilon\$ (eV)"
    elseif e_unit_type == :ω₀
        label_A = "\$A_\\mathbf{k}\\ [1 / \\omega_0]\$"
        label_e = "\$\\varepsilon\\ [\\omega_0]\$"
    end

    cbar.ax.set_ylabel(label_A; labelpad = 2)
    plotaxes_A_2d[1].set_ylabel(label_e)
    for ax in plotaxes_A_2d
        ax.set_ylim([ymin, ymax])
    end
    
    if plotaxes_A_1d !== nothing
        plotaxes_A_1d.set_ylim([ymin, ymax])
        plotaxes_A_1d.set_xlabel(label_A; labelpad = 0)
        plotaxes_A_1d.set_xscale("log")
    end

    return
end

function _plot_spectral_1d(data, plotaxes_A_1d; e_unit_type, ik0 = nothing)
    (; As_dict, ωs_plot, xplot, es_plot, es_onshell, ω_LO, e_ref, ib_rng, Tlist) = data
    if ik0 === nothing
        ik0 = findfirst(xplot .≈ 0)
    end

    if e_unit_type == :eV
        e_unit = unit_to_aru(:eV)
    elseif e_unit_type == :ω₀
        e_unit = 1.0
    else
        error("Invalid e_unit_type $e_unit_type")
    end
    _e_to_eV(e) = (e - e_ref) / e_unit

    for (i, key) in enumerate(["FM", "Cumulant", "scFM"])
        As = As_dict[key]

        z = dropdims(sum(As[:, ib_rng, :, iT], dims=2), dims=2) .* e_unit

        c = [color1, color2, color3][i]

        label = key
        # if label == "FM"
        #     label == "DM"
        # end
        # key == "Cumulant" && (label = "Cum.")
        # ls = [":", "-.", "-"][i]
        dashes = [(3, 1, 1, 1), (1, 0), (1, 1)][i]

        plotaxes_A_1d.plot(_e_to_eV.(ωs_plot), z[:, ik0]; c, label, dashes)

        if key == "scFM"
            e0 = ωs_plot[argmax(z[:, ik0])]

            plotaxes_A_1d.axvline.(_e_to_eV.([0.] .+ e0); c = color3, lw=0.8, ls="-")
            if Tlist[iT] > 0
                plotaxes_A_1d.axvline.(_e_to_eV.([ω_LO, -ω_LO] .+ e0); c = color3, lw=0.8, ls="--")
            else
                plotaxes_A_1d.axvline.(_e_to_eV.([ω_LO] .+ e0); c = color3, lw=0.8, ls="--")
            end
        end
    end

    if e_unit_type == :eV
        label_A = "\$A\\ (1 / \\mathrm{eV})\$"
        label_e = "\$\\varepsilon\$ (eV)"
    elseif e_unit_type == :ω₀
        label_A = "\$A\\ [1 / \\omega_0]\$"
        label_e = "\$\\varepsilon\\ [\\omega_0]\$"
    end
    plotaxes_A_1d.set_xlabel(label_e; labelpad = 0)
    plotaxes_A_1d.set_ylabel(label_A; labelpad = 0)

    return
end
