begin
    function get_xks_on_grid(ngrid)
        nk = ngrid[1]
        @assert ngrid == (nk, nk, 1)
        xks_on_grid = vcat(
            [Vec3(x, x, 0) for x in range(-div(nk, 3), 0, length=div(nk, 3)+1) ./ nk],
            [Vec3(x, 0, 0) for x in range(0, div(nk, 2), length=div(nk, 2)+1) ./ nk][2:end],
        )
        xplot_on_grid = norm.(Ref(model.recip_lattice) .* xks_on_grid) .* sign.(getindex.(xks_on_grid, 1))
        (; xks_on_grid, xplot_on_grid)
    end

    nk = 180

    wfpt = true

    nband = model.nw
    ib_rng = 1:1

    xks_plot = vcat(
        [Vec3(x, x, 0) for x in range(-1/3, 0., length=101)],
        [Vec3(x, 0, 0) for x in range(0., 0.5, length=101)][2:end],
    )
    es_plot = compute_eigenvalues_el(model, ElectronPhonon.Kpoints(xks_plot))
    xplot = norm.(Ref(model.recip_lattice) .* xks_plot) .* sign.(getindex.(xks_plot, 1))

    ωs_plot = range(-0.6, 0.4, step=0.001) .* unit_to_aru(:eV) .+ e_VBM

    postfix = "doping1.wfpt$wfpt.nk$nk"

    folder_output = "$folder/../4_julia/data/wide_window"
    filenames = [
        "$folder_output/data_Dyson.smearing_5meV.$postfix.jld2",
        "$folder_output/data_scFM.$postfix.jld2",
        "$folder_output/data_Dyson.smearing_5meV.$postfix.jld2",
        # "$folder_output/data_Dyson.smaering_sc.$postfix.jld2",
    ]

    μlist_dict = Dict()


    # Load self-consistent spectral function
    As_list = []
    Σs_list = []
    occ = nothing
    for filename in filenames
        As_list_, Σs_list_, occ = load_and_interpolate_spectral_function_1d_itp([filename], ωs_plot, xks_plot, xplot, get_xks_on_grid, nband, es_plot; ib_rng)
        push!(As_list, As_list_[1])
        push!(Σs_list, Σs_list_[1])
    end

    As_dict = Dict("FM" => As_list[1], "scFM" => As_list[2], "FM_scγ" => As_list[3]);
    Σs_dict = Dict("FM" => Σs_list[1], "scFM" => Σs_list[2], "FM_scγ" => Σs_list[3]);
    μlist_dict["FM"] = load(filenames[1], "occ").μlist
    μlist_dict["scFM"] = load(filenames[2], "occ").μlist
    μlist_dict["FM_scγ"] = load(filenames[3], "occ").μlist
    Tlist = load(filenames[1], "occ").Tlist
    nocc = length(μlist_dict["FM"])


    # Load cumulant spectral function
    postfix = "n0.0.wfpt$wfpt.nk$nk"
    filename = "$folder_output/data_cumulant_1d.eta5meV.$postfix.jld2"

    xks_itp = load(filename, "xks_cum")
    xplot_itp = norm.(Ref(model.recip_lattice) .* xks_itp) .* sign.(getindex.(xks_itp, 1))
    key = "Cumulant"

    μlist_dict["RSPT"] = load(filename, "occ").μlist
    μlist_dict["Cumulant"] = load(filename, "occ").μlist

    As_data = load(filename, "As_cum")
    ωs_data = load(filename, "ωs_cum")
    es_onshell = load(filename, "es_onshell")
    # Σs_rest_offset = real.(load(filename, "Σs_rest_offset"))
    Σs_rest_offset = zeros(nocc)

    for iT in 1:nocc
        es_onshell[:, :, iT] .+= Σs_rest_offset[iT]
    end

    As_cum = zeros(length(ωs_plot), model.nw, length(xks_plot), nocc)

    for iT in 1:nocc
        for ib in 1:model.nw
            As_itp = linear_interpolation((ωs_data, xplot_itp), As_data[:, ib, :, iT]; extrapolation_bc = 0.0)

            @views for ik in eachindex(xks_plot)
                As_cum[:, ib, ik, iT] .= @. As_itp.(ωs_plot .- Σs_rest_offset[iT], xplot[ik])
            end
        end
    end
    As_dict[key] = As_cum

    # Compute occupation
    spectral_sum_dict = Dict()
    for key in keys(As_dict)
        As = As_dict[key]
        dω = ωs_plot[2] - ωs_plot[1]
        spectral_sum_dict[key] = dropdims(sum(As, dims=1), dims=1) .* dω
    end

    data_InSe = (; Tlist, As_dict, Σs_dict, ωs_plot, xks_plot, xplot, es_plot,
                  es_onshell, spectral_sum_dict, ω_LO = ω_LO, e_ref = e_VBM, ib_rng, μlist_dict);
end;


begin
    # Read ARPES data
    data1 = readdlm("data_ARPES/Fig2a ARPES 1LGM.txt")
    data2 = readdlm("data_ARPES/Fig2a ARPES 1LGK.txt")
    data2[.!isa.(data2, Float64)] .= 0.0
    data2 = Float64.(data2)

    # Subtract background
    data1 .-= sum(data1[end-5:end, :]) / size(data1, 2) / 6
    data2 .-= sum(data2[end-5:end, :]) / size(data2, 2) / 6
    data1 ./= (maximum(data1) - minimum(data1))
    data2 ./= (maximum(data2) - minimum(data2))

    # data2: K-Γ-K, ik = 293 is Γ
    # data1: M-Γ-M, ik = 202 is Γ
    z_ARPES = hcat(data2[:, 1:293], data1[:, 202:end][:, 2:end])

    k1_K = 1.36947 / unit_to_aru(:Å)
    k1_M = 1.03567 / unit_to_aru(:Å)

    e1 = 6.25
    e2 = e1 + -0.021 * (239 - 1)
    # Minus sign to energy because ARPES measures binding energy
    e1 *= -1
    e2 *= -1
    e_offset = -2.25
    e1 -= e_offset
    e2 -= e_offset

    xplot_ARPES = vcat(range(-k1_K, 0, length = size(data2[:, 1:293], 2)),
                       range(0, k1_M, length = size(data1[:, 202:end], 2))[2:end])
    extent_ARPES = [-k1_K, k1_M, e1, e2]

    # Normalize to satisfy ∫ A(w) dw = 1

    es_limit = map(axes(z_ARPES, 2)) do ik
        k = xplot_ARPES[ik]
        if k < -0.4
            (-1.2, -0.1)
        elseif k < -0.2
            (-1.2 + (k + 0.4) * 0.9 / 0.2, -0.1 + (k + 0.4) * 0.6 / 0.2)
        elseif k < 0.0
            (-0.3 + (k + 0.2) * 0.0 / 0.2, 0.5 + (k + 0.2) * 0.0 / 0.2)
        elseif k < 0.2
            (-0.3 + (k - 0.0) * -0.0 / 0.2, 0.5 + (k - 0.0) * -0.0 / 0.2)
        elseif k < 0.4
            (-0.3 + (k - 0.2) * -0.9 / 0.2, 0.5 + (k - 0.2) * -0.6 / 0.2)
        else
            (-1.2, -0.1)
        end
    end
    es_limit = stack(es_limit)

    # Normalize to have ∫ A(w) dw = 1
    z_ARPES[z_ARPES .<= 0] .= 0

    es_ARPES = range(e1, e2, length = size(z_ARPES, 1))
    de = diff(es_ARPES)[1]
    for ik in axes(z_ARPES, 2)
        inds = es_limit[1, ik] .<= es_ARPES .<= es_limit[2, ik]
        z_ARPES[:, ik] ./= sum(z_ARPES[inds, ik]) * de
    end

    kwargs_A = (; origin="lower", aspect="auto", cmap="Greys", vmin = 0., vmax = 3.5)
    inds = xplot_ARPES .<= 0
    extent_ = [extent_ARPES[1], 0, extent_ARPES[3], extent_ARPES[4]]
    img = imshow(z_ARPES[:, inds] ; extent = extent_, origin="lower", aspect="auto", kwargs_A...)

    inds = xplot_ARPES .>= 0
    extent_ = [0, extent_ARPES[2], extent_ARPES[3], extent_ARPES[4]]
    img = imshow(z_ARPES[:, inds]; extent = extent_, origin="lower", aspect="auto", kwargs_A...)

    colorbar()
    plot(xplot_ARPES, es_limit[1, :], "k--")
    plot(xplot_ARPES, es_limit[2, :], "k--")
    axvline.([-0.38, 0.35])
    fig = gcf(); display(fig); close(fig)

    ymax =  1.0
    ymin = -1.0
end;
