include("0_common.jl")

begin
    lw = 1

    left   = 0.13
    right  = 0.96
    bottom = 0.11
    top    = 0.96

    fig = figure(figsize=(3.4, 1.6))
    gridspec = fig.add_gridspec(; nrows = 1, ncols = 1, left, right, top, bottom)
    ax = fig.add_subplot(gridspec[1, 1])

    kpts, plot_xdata = high_symmetry_kpath(model; is_2d = true, kline_density = 400)
    es_wannier = compute_eigenvalues_el(model, kpts)

    data = readdlm("../0_bands/InSe.bands.dat.gnu")
    nk = findfirst(data[2:end, 1] .== 0)
    xs = data[1:nk, 1]
    xs .*= plot_xdata.x[end] ./ xs[end]
    es = reshape(data[:, 2], nk, :) .- e_VBM / unit_to_aru(:eV)
    lines1, = plot(xs, es; c="k", lw)

    lines2, = plot(plot_xdata.x, _e_to_eV.(es_wannier'); c="r", ls = "--", lw)

    leg = ax.legend([lines1, lines2], ["DFT", "Wannier"], loc = "lower right", framealpha = 1,)
    leg.get_frame().set_linewidth(lw/2)

    ax.axhline(0; c = "silver", lw, zorder = 1)
    ax.axvline.(plot_xdata.xticks; c="silver", lw, ls="--", zorder = 1)
    ax.set_xticks(plot_xdata.xticks)
    ax.set_xticklabels(plot_xdata.xlabels)
    ax.set_xlim(extrema(plot_xdata.x))
    ax.set_ylabel("\$\\varepsilon_{n\\mathbf{k}} - \\varepsilon_{\\mathrm{VBM}}\$ (eV)")
    ax.set_ylim(-2, 0.5)
    ax.set_yticks([-2, -1, 0])

    do_savefig(fig, "./figures/InSe_bands"; transparent = false)

    display(fig); close(fig)
end
