include("0_common.jl")

function compute_Σ_qp(filename; on_shell = false)
    kpts, Σs, eks, ωs, ωs_dense, occ = load(filename, "kpts", "Σs", "eks", "ωs", "ωs_dense", "occ")
    nband = size(eks, 1)

    Σ_qp = zeros(ComplexF64, nband, kpts.n, length(occ))
    for iT in 1:length(occ), ib in 1:nband, ik in 1:kpts.n
        Σs_itp = linear_interpolation(ωs, Σs[:, ib, ik, iT])
        As = @. -imag(1 / (ωs_dense - eks[ib, ik] - Σs_itp(ωs_dense))) / π
        if on_shell
            iω = argmin(abs.(ωs .- eks[ib, ik]))
        else
            iω = argmax(As)
        end
        Σ_qp[ib, ik, iT] = Σs_itp(ωs_dense[iω])
    end

    eks, Σ_qp
end

begin
    folder_output = "$folder/../4_julia/data"
    ind_doping = 7

    nk_bte = 180
    nk_bubble = 180

    wfpt = true

    mob_serta_SI_dict, mob_bte_SI_dict, Tlist_bte = load("$folder_output/data_bte.longrange.quadrupole.jld", "mobility_serta_SI_dict", "mobility_bte_SI_dict", "Tlist")
    Tlist_bte = Tlist_bte[1:7]
    mob_serta_SI_dict = Dict([(k, v[1:7]) for (k, v) in mob_serta_SI_dict])
    mob_bte_SI_dict = Dict([(k, v[1:7]) for (k, v) in mob_bte_SI_dict])

    mob_bubble_SI_dict, Tlist_bubble = load("$folder_output/data_bubble.jld", "mob_SI_bubble_dict", "Tlist")
    ac_mob_SI_bubble_dict, ac_Ωlist = load("$folder_output/data_bubble.jld", "ac_mob_SI_bubble_dict", "Ωs")
    ac_jdos_dict = load("$folder_output/data_bubble.jld", "ac_jdos_dict")

    @assert Tlist_bubble == Tlist_bte

    Tlist = Tlist_bte ./ unit_to_aru(:K)
    ac_Ωlist = ac_Ωlist ./ unit_to_aru(:meV)
end;

begin
    FM_scγ = false

    fig = figure(figsize=(3.4, 3.5))

    left   = 0.14
    right  = 0.94
    bottom = 0.10
    top    = 0.96

    gs1 = fig.add_gridspec(; ncols = 1, nrows = 4, hspace = 0.05, left, right, top, bottom, height_ratios = [1.0, 1., 0.45, 2.])
    axes_mob = [fig.add_subplot(gs1[i]) for i in 1:2]
    ax_scat = fig.add_subplot(gs1[4])

    ms = 6
    lw = 1
    # DC mobility
    sca(axes_mob[2])

    # FM mobility: plot on upper axis
    key = (nk_bubble, wfpt, ind_doping, "data_Dyson.smearing_5meV")
    label = FM_scγ ? "Bubble-\$G_0 D_0\$, \$\\eta=5\\ \\mathrm{meV}\$" : "Bubble-\$G_0 D_0\$"
    axes_mob[1].plot(Tlist, mob_bubble_SI_dict[key], "x-"; c = c_FM, label, ms, lw)
    axes_mob[2].plot(Tlist, mob_bubble_SI_dict[key], "x-"; c = c_FM, label, ms, lw)

    if FM_scγ
        key = (nk_bubble, wfpt, ind_doping, "data_Dyson.smearing_sc")
        plot(Tlist, mob_bubble_SI_dict[key], "s-"; c = "C2", label = "Bubble-\$G_0 D_0\$, \$\\eta=\\gamma^\\mathrm{sc}\$", ms, mew = 0, lw)
    end

    key = (nk_bubble, wfpt, ind_doping, "data_scFM")
    plot(Tlist, mob_bubble_SI_dict[key], "o-"; c = c_scFM, label = "Bubble-sc\$GD_0\$", ms, mew = 0, lw)
    println("Ratio Bubble / BTE = ", mob_bubble_SI_dict[key] ./ mob_bte_SI_dict[(nk_bte, ind_doping)])

    if !FM_scγ
        plot(Tlist, mob_bte_SI_dict[(nk_bte, ind_doping)], "v--"; fillstyle = "none", ms, mew=0.5, c = c_BTE, label = "BTE-full", lw)
        plot(Tlist, mob_serta_SI_dict[(nk_bte, ind_doping)], "^:"; fillstyle = "none", ms, mew=0.5, c = c_SERTA, label = "BTE-SERTA", lw)
    end

    kwargs_legend = (; frameon=false, labelspacing=0.05, handletextpad=0.3, handlelength = 1.3, title_fontsize = fontsize, borderpad = 0., alignment = "center")
    axes_mob[2].legend(; kwargs_legend..., bbox_to_anchor = (0., 0., 1., 1.15), loc = "upper right")
    axes_mob[2].set_xlabel("\$T\\ \\mathrm{(K)}\$"; labelpad=0)
    axes_mob[2].text(-0.12, +0.1, "DC mobility (cm\$^2\$/Vs)"; transform=axes_mob[2].transAxes, ha="center", va="bottom", rotation = 90)

    # Generate skipped y axis
    axes_mob[1].set_ylim([46, 58])
    axes_mob[1].set_yticks([50, 55])
    if FM_scγ
        axes_mob[2].set_ylim([0, 14])
    else
        axes_mob[2].set_ylim([0, 8.5])
        axes_mob[2].set_yticks([0, 4, 8])
    end

    # hide the spines between ax and ax2
    ax1, ax2 = axes_mob
    ax1.spines.bottom.set_visible(false)
    ax2.spines.top.set_visible(false)
    ax1.xaxis.tick_top()
    ax1.tick_params(labeltop=false)  # don't put tick labels at the top
    ax2.xaxis.tick_bottom()

    # Slanted lines
    d = .5  # proportion of vertical to horizontal extent of the slanted line
    kwargs = (; marker=[(-1, -d), (1, d)], ms=5, ls="none", c="k", mec="k", mew=0.5, clip_on=false)
    ax1.plot([0, 1], [0, 0]; transform=ax1.transAxes, kwargs...)
    ax2.plot([0, 1], [1.02, 1.02]; transform=ax2.transAxes, kwargs...)



    # plotaxes[1].set_ylim([0, 14])

    # Scattering rate
    sca(ax_scat)
    iT = 3

    on_shell = false
    if FM_scγ
        for i in 1:3
            # 1: FM, 5meV + on shell
            # 2: FM, scγ + on shell
            # 3: scFM + on shell
            if i == 1
                filename = "$folder_output/data_Dyson.smearing_5meV.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            elseif i == 2
                filename = "$folder_output/data_Dyson.smearing_sc.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            elseif i == 3
                filename = "$folder_output/data_scFM.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            end
            es, Σ_qp = compute_Σ_qp(filename; on_shell)

            label = [
                "Bubble-\$G_0 D_0\$, \$\\eta = 5\\,\\mathrm{meV}\$",
                "Bubble-\$G_0 D_0\$, \$\\eta = \\gamma^\\mathrm{sc}\$",
                "Bubble-sc\$GD_0\$",
            ][i]
            zorder = i == 1 ? 10 : 2
            marker = ["x", "s", "o"][i]
            c = [c_FM, "C2", c_scFM][i]
            inds = 1:3:size(es, 2)
            inds = 1:1:size(es, 2)
            plot(_e_to_eV.(es[1, inds]), .-imag.(Σ_qp[1, inds, iT]) .* 2 ./ unit_to_aru(:meV); label, ms, marker, lw = 0, mew = 0.5, c, zorder)
        end
    else
        for i in 1:2
            # 1: FM + off shell
            # 2: scFM + off shell
            # 3: FM + on shell
            if i == 2
                filename = "$folder_output/data_scFM.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            else
                filename = "$folder_output/data_Dyson.smearing_5meV.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            end
            es, Σ_qp = compute_Σ_qp(filename; on_shell)

            label = [
                "Bubble-\$G_0 D_0\$",
                "Bubble-sc\$GD_0\$",
                "BTE",
            ][i]
            zorder = i == 1 ? 10 : 2
            marker = ["x", "o", "v"][i]
            c = [c_FM, c_scFM, c_BTE][i]
            fillstyle = i == 3 ? "none" : "full"
            inds = 1:3:size(es, 2)
            plot(_e_to_eV.(es[1, inds]), .-imag.(Σ_qp[1, inds, iT]) .* 2 ./ unit_to_aru(:meV); label, ms, marker, lw = 0, mew = 0.5, c, fillstyle, zorder)
        end

        es_dict, γ_dict = load("$folder_output/data_bte.longrange.quadrupole.jld", "es_dict", "γs_dict")
        inds = 1:3:size(es_dict[(nk_bte, ind_doping)], 1)
        plot(_e_to_eV.(es_dict[(nk_bte, ind_doping)])[inds], γ_dict[(nk_bte, ind_doping)][iT][inds] ./ unit_to_aru(:meV); label = "BTE", c = c_BTE, ms, marker = "v", lw = 0, fillstyle = "none", mew = 0.5)
    end


    xlim([-0.2, 0.])
    xticks([-0.2, -0.15, -0.1, -0.05, 0.])
    xlabel("\$\\varepsilon_{n\\mathbf{k}} - \\varepsilon_\\mathrm{VBM} \\ (\\mathrm{eV})\$"; labelpad=0)
    ylabel("\$\\tau_{n\\mathbf{k}}^{-1}\$ (meV)"; labelpad=0)
    if FM_scγ
        ylim([0, 190])
    else
        yscale("log")
        ax_scat.legend(; kwargs_legend...)
    end



    fig.align_xlabels([axes_mob[2], ax_scat])

    for (iax, ax) in enumerate([axes_mob[1], ax_scat])
        x = [0.08, 0.01][iax]
        y = [0.96, 0.98][iax]
        x = 0.01
        y = 0.98
        label = ["", " \$T = 100~\\mathrm{K}\$"][iax]
        ax.text(x, y, "($(('a':'z')[iax]))" * label; transform=ax.transAxes, ha="left", va="top")
    end

    if FM_scγ
        do_savefig(fig, "./figures/InSe_conductivity_dc_smearing"; transparent = false)
    else
        do_savefig(fig, "./figures/InSe_conductivity_dc"; transparent = false)
    end

    display(fig); close(fig)
end

begin
    FM_scγ = true

    fig = figure(figsize=(3.4, 3.5))

    left   = 0.14
    right  = 0.94
    bottom = 0.10
    top    = 0.96

    gs1 = fig.add_gridspec(; ncols = 1, nrows = 2, hspace = 0.25, left, right, top, bottom, height_ratios = [1.0, 1.])
    ax_mob = fig.add_subplot(gs1[1])
    ax_scat = fig.add_subplot(gs1[2])

    ms = 6
    lw = 1
    # DC mobility
    sca(ax_mob)

    # FM mobility: plot on upper axis
    key = (nk_bubble, wfpt, ind_doping, "data_Dyson.smearing_5meV")
    label = FM_scγ ? "Bubble-\$G_0 D_0\$, \$\\eta=5\\ \\mathrm{meV}\$" : "Bubble-\$G_0 D_0\$"
    ax_mob.plot(Tlist, mob_bubble_SI_dict[key], "x-"; c = c_FM, label, ms, lw)

    if FM_scγ
        key = (nk_bubble, wfpt, ind_doping, "data_Dyson.smearing_sc")
        ax_mob.plot(Tlist, mob_bubble_SI_dict[key], "s-"; c = "C2", label = "Bubble-\$G_0 D_0\$, \$\\eta=\\gamma^\\mathrm{sc}\$", ms, mew = 0, lw)
    end

    key = (nk_bubble, wfpt, ind_doping, "data_scFM")
    ax_mob.plot(Tlist, mob_bubble_SI_dict[key], "o-"; c = c_scFM, label = "Bubble-sc\$GD_0\$", ms, mew = 0, lw)
    println("Ratio Bubble / BTE = ", mob_bubble_SI_dict[key] ./ mob_bte_SI_dict[(nk_bte, ind_doping)])

    if !FM_scγ
        ax_mob.plot(Tlist, mob_bte_SI_dict[(nk_bte, ind_doping)], "v--"; fillstyle = "none", ms, mew=0.5, c = c_BTE, label = "BTE-full", lw)
        ax_mob.plot(Tlist, mob_serta_SI_dict[(nk_bte, ind_doping)], "^:"; fillstyle = "none", ms, mew=0.5, c = c_SERTA, label = "BTE-SERTA", lw)
    end

    kwargs_legend = (; frameon=false, labelspacing=0.05, handletextpad=0.3, handlelength = 1.3, title_fontsize = fontsize, borderpad = 0., alignment = "center")
    ax_mob.legend(; kwargs_legend..., bbox_to_anchor = (0., 0., 1., 0.6), loc = "upper right")
    ax_mob.set_xlabel("\$T\\ \\mathrm{(K)}\$"; labelpad=0)
    ax_mob.set_ylabel("DC mobility (cm\$^2\$/Vs)")

    ax_mob.set_ylim([0, 65])



    # plotaxes[1].set_ylim([0, 14])

    # Scattering rate
    sca(ax_scat)
    iT = 3

    on_shell = false
    if FM_scγ
        for i in 1:3
            # 1: FM, 5meV + on shell
            # 2: FM, scγ + on shell
            # 3: scFM + on shell
            if i == 1
                filename = "$folder_output/data_Dyson.smearing_5meV.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            elseif i == 2
                filename = "$folder_output/data_Dyson.smearing_sc.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            elseif i == 3
                filename = "$folder_output/data_scFM.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            end
            es, Σ_qp = compute_Σ_qp(filename; on_shell)

            label = [
                "Bubble-\$G_0 D_0\$, \$\\eta = 5\\,\\mathrm{meV}\$",
                "Bubble-\$G_0 D_0\$, \$\\eta = \\gamma^\\mathrm{sc}\$",
                "Bubble-sc\$GD_0\$",
            ][i]
            zorder = i == 1 ? 10 : 2
            marker = ["x", "s", "o"][i]
            c = [c_FM, "C2", c_scFM][i]
            inds = 1:3:size(es, 2)
            inds = 1:1:size(es, 2)
            plot(_e_to_eV.(es[1, inds]), .-imag.(Σ_qp[1, inds, iT]) .* 2 ./ unit_to_aru(:meV); label, ms, marker, lw = 0, mew = 0.5, c, zorder)
        end
    else
        for i in 1:2
            # 1: FM + off shell
            # 2: scFM + off shell
            # 3: FM + on shell
            if i == 2
                filename = "$folder_output/data_scFM.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            else
                filename = "$folder_output/data_Dyson.smearing_5meV.doping$ind_doping.wfpt$wfpt.nk$nk_bubble.jld2"
            end
            es, Σ_qp = compute_Σ_qp(filename; on_shell)

            label = [
                "Bubble-\$G_0 D_0\$",
                "Bubble-sc\$GD_0\$",
                "BTE",
            ][i]
            zorder = i == 1 ? 10 : 2
            marker = ["x", "o", "v"][i]
            c = [c_FM, c_scFM, c_BTE][i]
            fillstyle = i == 3 ? "none" : "full"
            inds = 1:3:size(es, 2)
            plot(_e_to_eV.(es[1, inds]), .-imag.(Σ_qp[1, inds, iT]) .* 2 ./ unit_to_aru(:meV); label, ms, marker, lw = 0, mew = 0.5, c, fillstyle, zorder)
        end

        es_dict, γ_dict = load("$folder_output/data_bte.longrange.quadrupole.jld", "es_dict", "γs_dict")
        inds = 1:3:size(es_dict[(nk_bte, ind_doping)], 1)
        plot(_e_to_eV.(es_dict[(nk_bte, ind_doping)])[inds], γ_dict[(nk_bte, ind_doping)][iT][inds] ./ unit_to_aru(:meV); label = "BTE", c = c_BTE, ms, marker = "v", lw = 0, fillstyle = "none", mew = 0.5)
    end


    xlim([-0.2, 0.])
    xticks([-0.2, -0.15, -0.1, -0.05, 0.])
    xlabel("\$\\varepsilon_{n\\mathbf{k}} - \\varepsilon_\\mathrm{VBM} \\ (\\mathrm{eV})\$"; labelpad=0)
    ylabel("\$\\tau_{n\\mathbf{k}}^{-1}\$ (meV)"; labelpad=0)
    if FM_scγ
        ylim([0, 190])
    else
        yscale("log")
        ax_scat.legend(; kwargs_legend...)
    end



    fig.align_ylabels([ax_mob, ax_scat])

    for (iax, ax) in enumerate([ax_mob, ax_scat])
        x = [0.08, 0.01][iax]
        y = [0.96, 0.98][iax]
        x = 0.01
        y = 0.98
        label = ["", " \$T = 100~\\mathrm{K}\$"][iax]
        ax.text(x, y, "($(('a':'z')[iax]))" * label; transform=ax.transAxes, ha="left", va="top")
    end

    if FM_scγ
        do_savefig(fig, "./figures/InSe_conductivity_dc_smearing"; transparent = false)
    else
        do_savefig(fig, "./figures/InSe_conductivity_dc"; transparent = false)
    end

    display(fig); close(fig)
end

for FM_scγ in [false, true]

    fig = figure(figsize=(3.4, 1.6))

    left   = 0.10
    right  = 0.96
    bottom = 0.20
    top    = 0.96

    gridspec = fig.add_gridspec(; ncols = 2, nrows = 1, wspace = 0.15, left, right, top, bottom)
    plotaxes = [fig.add_subplot(gridspec[i]) for i in 1:2]

    if !FM_scγ
        x = 0.55
        y = 0.4
        ax_inset = plotaxes[1].inset_axes([x, y, 1-x, 1-y-0.13])
        ax_inset.patch.set_facecolor("none")
        ax_inset.tick_params(axis="x", which="major", top = false)
        ax_inset.tick_params(axis="y", which="major", right = false)
    end

    # AC mobility
    iTlist = [1, 3, 4, 5, 6, 7]
    lw = 1
    for (i, iT) in enumerate(iTlist)
        c = PyPlot.get_cmap("plasma")(((i-1) / (length(iTlist)-1)) * 0.8)

        if FM_scγ
            sc = "data_Dyson.smearing_sc"
        else
            sc = "data_Dyson.smearing_5meV"
        end
        T_Kelvin = Tlist[iT]
        label = @sprintf "%.0f" T_Kelvin

        key = (nk_bubble, wfpt, ind_doping, sc)
        plotaxes[1].plot(ac_Ωlist, ac_mob_SI_bubble_dict[key][iT, :], "-"; c, label, lw)

        # Arrow indicating maximum
        iΩ_max = findlast(diff(ac_mob_SI_bubble_dict[key][iT, :]) .> 0)
        if iΩ_max !== nothing && ac_Ωlist[iΩ_max] > 100
            Ω_max = ac_Ωlist[iΩ_max]
            mob_max = ac_mob_SI_bubble_dict[key][iT, iΩ_max]

            plotaxes[1].annotate("", xytext=(Ω_max, mob_max + 0.8), xy=(Ω_max, mob_max), arrowprops = Dict("arrowstyle" => "->", "color" => c, "linewidth" => 1.5))
        end


        if !FM_scγ
            ax_inset.plot(ac_Ωlist, ac_mob_SI_bubble_dict[key][iT, :], "-"; c, label, lw)
        end

        key = (nk_bubble, wfpt, ind_doping, "data_scFM")
        plotaxes[2].plot(ac_Ωlist, ac_mob_SI_bubble_dict[key][iT, :], "-"; c, label, lw)
    end

    if !FM_scγ
        ax_inset.set_xlim([0, 9])
        ax_inset.set_ylim([0, 60])
        ax_inset.set_yticks([0, 25, 50])
        # y0 = 1.2
        # ax_inset.set_xlim([100, 300])
        # ax_inset.set_ylim([0, y0])
        # ax_inset.set_yticklabels([])
        # ax_inset.set_xticklabels([])
        # plotaxes[1].plot([100, 300], [y0, y0], "-"; lw = 0.7, c = "k", ls = "-", zorder = 10)
        # plotaxes[1].plot([100, 100], [0, y0], "-"; lw = 0.7, c = "k", ls = "-", zorder = 10)
        # plotaxes[1].plot([100, 180], [y0, 9], "-"; lw = 0.7, c = "k", ls = "--", zorder = 10)
    end

    kwargs_legend = (; frameon=false, labelspacing=0.05, handletextpad=0.3, handlelength = 1.7, title_fontsize = fontsize, borderpad = 0., alignment = "center", columnspacing = 0.5)
    plotaxes[2].legend(; title = "\$T~\$(K)", kwargs_legend..., loc = "upper right", ncol = 2, bbox_to_anchor = (1.0, 0.85))

    plotaxes[1].set_ylim([0, 4.5])
    plotaxes[2].set_ylim([0, 4.5])
    plotaxes[2].set_yticklabels([])

    plotaxes[1].set_ylabel("AC mobility (cm\$^2\$/Vs)")

    for ax in plotaxes
        ax.set_xlim([0, 300])
        ax.set_xlabel("\$\\Omega\\ (\\mathrm{meV})\$"; labelpad = 2)
    end
    fig.align_xlabels(plotaxes)

    for (iax, ax) in enumerate(plotaxes)
        if FM_scγ
            label = ["Bubble-\$G_0 D_0\$,\n      \$\\eta=\\gamma^{\\mathrm{sc}}\$", "Bubble-sc\$GD_0\$"][iax]
            if iax == 1
                x, y = 0.15, 0.97
            else
                x, y = 0.1, 0.97
            end
        else
            label = ["Bubble-\$G_0 D_0\$", "Bubble-sc\$GD_0\$"][iax]
            x, y = 0.1, 0.97
        end
        ax.text(x, y, "($(('a':'z')[iax]))" * " $label"; transform=ax.transAxes, ha="left", va="top")#
        # , bbox = Dict("facecolor" => "w", "edgecolor" => "none", "pad" => 0.0), zorder = 2)
    end

    if FM_scγ
        do_savefig(fig, "./figures/InSe_conductivity_ac_allT_smearing"; transparent = false)
    else
        do_savefig(fig, "./figures/InSe_conductivity_ac_allT"; transparent = false)
    end

    display(fig); close(fig)
end

begin
    fig = figure(figsize=(3.4, 1.5))

    left   = 0.15
    right  = 0.98
    bottom = 0.20
    top    = 0.96

    gridspec = fig.add_gridspec(; ncols = 2, nrows = 2, wspace = 0.10, hspace = 0.08, left, right, top, bottom)
    plotaxes_FM = [fig.add_subplot(gridspec[1, i]) for i in 1:2]
    plotaxes = [fig.add_subplot(gridspec[2, i]) for i in 1:2]

    ms = 3
    lw = 1

    iTlist = [1, 3]

    for (iax, iT) in enumerate(iTlist)
        sca(plotaxes[iax])

        sc = "data_Dyson.smearing_5meV"
        nklist = sort(unique!([x[1] for x in keys(mob_bubble_SI_dict) if x[2:4] == (wfpt, ind_doping, sc)]))
        plotaxes_FM[iax].plot(nklist, [mob_bubble_SI_dict[(nk, wfpt, ind_doping, sc)][iT] for nk in nklist], "x-"; c = c_FM, label = "Bubble-\$G_0 D_0\$", ms, lw)

        sc = "data_scFM"
        nklist = sort(unique!([x[1] for x in keys(mob_bubble_SI_dict) if x[2:4] == (wfpt, ind_doping, sc)]))
        plot(nklist, [mob_bubble_SI_dict[(nk, wfpt, ind_doping, sc)][iT] for nk in nklist], "o-"; c = c_scFM, label = "Bubble-sc\$GD_0\$", ms, lw)

        nklist = sort(unique!([x[1] for x in keys(mob_bte_SI_dict) if x[2] == ind_doping]))
        plot(nklist, [mob_bte_SI_dict[(nk, ind_doping)][iT] for nk in nklist], "v--"; fillstyle = "none", ms, mew=0.5, c = c_BTE, label = "BTE-full", lw)
        plot(nklist, [mob_serta_SI_dict[(nk, ind_doping)][iT] for nk in nklist], "^:"; fillstyle = "none", ms, mew=0.5, c = c_SERTA, label = "BTE-SERTA", lw)
    end

    for iax in 1:2
        iT = iTlist[iax]
        # hide the spines between ax1 and ax2
        ax1, ax2 = plotaxes_FM[iax], plotaxes[iax]
        ax1.spines.bottom.set_visible(false)
        ax2.spines.top.set_visible(false)
        ax1.xaxis.tick_top()
        ax1.tick_params(labeltop=false)  # don't put tick labels at the top
        ax2.xaxis.tick_bottom()

        # Slanted lines
        d = .5  # proportion of vertical to horizontal extent of the slanted line
        kwargs = (; marker=[(-1, -d), (1, d)], ms=5, ls="none", c="k", mec="k", mew=0.5, clip_on=false)
        ax1.plot([0, 1], [0, 0]; transform=ax1.transAxes, kwargs...)
        ax2.plot([0, 1], [1.02, 1.02]; transform=ax2.transAxes, kwargs...)

        ax1.set_xticks([30, 60, 120, 180])
        ax2.set_xticks([30, 60, 120, 180])
        ax1.set_xlim([30, 200])
        ax2.set_xlim([30, 200])

        ax1.set_ylim([45, 75])
        ax2.set_ylim([0, 8.5])
        ax2.set_yticks([0, 4, 8])

        if iax == 2
            ax1.set_yticklabels([])
            ax2.set_yticklabels([])
        end
    end

    kwargs_legend = (; frameon=false, labelspacing=0.05, handletextpad=0.3, handlelength = 1.7, title_fontsize = fontsize, borderpad = 0., alignment = "center")
    plotaxes_FM[2].legend(; kwargs_legend..., bbox_to_anchor = (0.18, 0., 1., 0.60), loc = "upper left")
    plotaxes[2].legend(; kwargs_legend..., bbox_to_anchor = (0.18, 0., 1., 1.25), loc = "upper left")

    # plotaxes[1].set_ylabel("dc mobility (cm\$^2\$/Vs)"; labelpad=0)
    plotaxes[1].text(-0.3, 0., "DC mobility (cm\$^2\$/Vs)"; rotation = 90, transform = plotaxes[1].transAxes, ha = "center", va = "bottom")

    for ax in plotaxes
        ax.set_xlabel("\$N_k\$"; labelpad=0)
    end
    fig.align_xlabels(plotaxes)

    for (iax, ax) in enumerate(plotaxes_FM)
        label = " T=$(round(Int, Tlist[iTlist[iax]])) K"
        ax.text(0.03, 0.93, "($(('a':'z')[iax]))" * label; transform=ax.transAxes, ha="left", va="top")
    end

    do_savefig(fig, "./figures/InSe_conductivity_conv"; transparent = false)

    display(fig); close(fig)
end


begin
    fig = figure(figsize=(3.4, 2.0))

    left   = 0.12
    right  = 0.96
    bottom = 0.14
    top    = 0.96

    gridspec = fig.add_gridspec(; ncols = 1, nrows = 1, wspace = 0.05, left, right, top, bottom)
    plotaxes = [fig.add_subplot(gridspec[i]) for i in 1:1]
    sca(plotaxes[1])

    nk = 60
    ms = 5
    lw = 1
    for (ic, longrange_mode) in enumerate([:dipole, :quadrupole])
        c = "C$(ic-1)"
        filename = "$folder_output/data_bte.longrange.$longrange_mode.jld"
        mob_serta_SI_dict_, mob_bte_SI_dict_, Tlist_bte_ = load(filename, "mobility_serta_SI_dict", "mobility_bte_SI_dict", "Tlist")
        if longrange_mode == :dipole
            label = "D"
        else
            label = "D+Q+A"
        end
        plot(Tlist_bte_ ./ unit_to_aru(:K), mob_serta_SI_dict_[(nk, ind_doping)], "^--"; ms, lw, mew = 0.5, c, label = "BTE-SERTA, $label", fillstyle = "none")
        plot(Tlist_bte_ ./ unit_to_aru(:K), mob_bte_SI_dict_[(nk, ind_doping)], "v-"; ms, lw, mew = 0.5, c, label = "BTE-full, $label")
    end

    mob_bte_Li2019 = [0.90186, 0.62095, 0.52916, 0.48849, 0.46313, 0.45700, 0.45095, 0.45396, 0.44795]
    Tlist_Li2019 = 100.0:50:500.0
    plot(Tlist_Li2019, mob_bte_Li2019, "kx"; ms, lw, mew = 0.5, label = "BTE-SERTA, Li et al (2019)", zorder = 3)

    mob_bte_Ponce2023 = [1.09370, 0.70759, 0.59451, 0.55897, 0.55250, 0.55735]
    Tlist_Ponce2023 = 100.0:50:350.0
    plot(Tlist_Ponce2023, mob_bte_Ponce2023, "o"; c = "gray", ms, lw, mew = 0.5, label = "BTE-full, Poncé et al (2023)")

    xlabel("\$T\$ (K)"; labelpad=0)
    ylabel("Mobility (cm\$^2\$/Vs)"; labelpad=0)
    # yscale("log")
    xlim([40, 310])
    ylim([0, 2.5])
    legend(; frameon = false)

    do_savefig(fig, "./figures/InSe_conductivity_comparison"; transparent = false)

    display(fig); close(fig)
end
