include("0_common.jl")

begin
    nk = 180
    wfpt = true
    ind_doping = 1
    folder_output = "$folder/../4_julia/data/doping_dependence"

    mob_SI_bubble_dict, nlist_SI, Tlist_Kelvin = load("$folder_output/data_bubble.jld", "mob_SI_bubble_dict", "nlist_SI", "Tlist_Kelvin")
    @info "T = $(Tlist_Kelvin[1])"

    ms = 5
    lw = 1

    # η_tag = "smearing_5meV"
    # η_tag = "smearing_1meV"

    for η_tag in ["smearing_5meV", "smearing_1meV"]

        if η_tag == "smearing_5meV"
            filename_bte = "$folder_output/data_bte.jld"
        else
            filename_bte = "$folder_output/data_bte.$η_tag.jld"
        end
        mobility_bte_SI_dict, mobility_serta_SI_dict, μlist_bte_dict = load(filename_bte, "mobility_bte_SI_dict", "mobility_serta_SI_dict", "μlist_dict")
        
        # Find cutoff density : μ_BTE(n_cutoff) = e_VBM
        μlist_itp = linear_interpolation(log10.(nlist_SI), μlist_bte_dict[(nk, ind_doping)])
        xs = range(extrema(log10.(nlist_SI))..., length = 1001)
        n_cutoff = 10.0 .^ xs[argmin(abs.(μlist_itp.(xs) .- e_VBM))]

        fig = figure(figsize=(3.4, 3.0))

        left   = 0.14
        right  = 0.98
        bottom = 0.11
        top    = 0.98
    
        gridspec = fig.add_gridspec(; ncols = 1, nrows = 1, wspace = 0.10, hspace = 0.08, left, right, top, bottom)
        ax = fig.add_subplot(gridspec[1, 1])
        sca(ax)

        key = (nk, wfpt, ind_doping, "data_Dyson.$η_tag.no_truncation")
        plot(nlist_SI, mob_SI_bubble_dict[key], "+:"; c = c_FM, ms, lw, label = "\$G_0 D_0\$")#, w/o trunc.")

        key = (nk, wfpt, ind_doping, "data_Dyson.$η_tag")
        plot(nlist_SI, mob_SI_bubble_dict[key], "x-"; c = c_FM, label = "\$G_0 D_0\$+trunc.", ms, lw)

        key = (nk, wfpt, ind_doping, "data_cumulant.$η_tag")
        plot(nlist_SI, mob_SI_bubble_dict[key], "s-"; c = c_Cum, label = "Cumulant", ms, mew = 0, lw)

        key = (nk, wfpt, ind_doping, "data_scFM")
        plot(nlist_SI, mob_SI_bubble_dict[key], "o-"; c = c_scFM, label = "sc\$G D_0\$", ms, mew = 0, lw)

        l1, = plot(nlist_SI, mobility_bte_SI_dict[(nk, ind_doping)], "v--"; fillstyle = "none", ms, mew=0.5, c = c_BTE, lw)
        l2, = plot(nlist_SI, mobility_serta_SI_dict[(nk, ind_doping)], "^:"; fillstyle = "none", ms, mew=0.5, c = c_SERTA, lw)

        # fill_between([n_cutoff, maximum(nlist_SI)], 4e-2, 1e3; fc = "k", alpha = 0.3)
        # fill_between([n_cutoff, maximum(nlist_SI)], 0, 6e-3; fc = "k", alpha = 0.3)


        # # legend(; frameon = false)
        # kwargs_legend = (; ncol = 2, borderpad = 0, columnspacing = 0.5, frameon = false, loc = "lower center", handlelength = 1.5)
        # leg1 = legend([l1, l2], ["Full", "SERTA"]; bbox_to_anchor = (0.13, -0.02, 1, 1), title = "BTE", kwargs_legend...)
        # leg2 = ax.legend(; frameon = false, bbox_to_anchor = (0.13, 0.12, 1, 1), title = "Bubble", kwargs_legend...)
        # ax.add_artist(leg1)
        # leg1._legend_box.sep = 2
        # leg2._legend_box.sep = 2

        # fill_between([n_cutoff, maximum(nlist_SI)], 2.2e-1, 1e3; fc = "silver")
        # fill_between([n_cutoff, maximum(nlist_SI)], 0, 4.5e-2; fc = "silver")
        # fill_between([n_cutoff * 3.5, maximum(nlist_SI)], 1e-2, 1e0; fc = "silver")
        # fill_between([n_cutoff, maximum(nlist_SI)], 2.2e-1, 1e5; fc = "silver")
        # fill_between([n_cutoff, maximum(nlist_SI)], 0, 3.5e-2; fc = "silver")
        # fill_between([n_cutoff * 3.0, maximum(nlist_SI)], 1e-2, 1e0; fc = "silver")
        # fill_between([n_cutoff * 1.7, maximum(nlist_SI)], 1e-2, 8.7e-2; fc = "silver")
        fill_between([n_cutoff, maximum(nlist_SI)], 0, 1e5; fc = "silver")

        # x = 0.58
        # kwargs_legend = (; borderpad = 0, columnspacing = 0.5, frameon = false, loc = "lower left", handlelength = 1.5, labelspacing = 0.1)
        # leg1 = legend([l1, l2], ["Full", "SERTA"]; bbox_to_anchor = (x, 0.0, 1, 1), ncol = 1, kwargs_legend...)
        # leg2 = ax.legend(; frameon = false, bbox_to_anchor = (x, 0.15, 1, 1), ncol = 1, kwargs_legend...)
        # ax.add_artist(leg1)
        # leg1._legend_box.sep = 2
        # leg2._legend_box.sep = 2
        # ax.text(x - 0.05, 0.22, "Bubble"; rotation = 90, transform = ax.transAxes)
        # ax.text(x - 0.05, 0.05, "BTE"; rotation = 90, transform = ax.transAxes)


        kwargs_legend = (; borderpad = 0, columnspacing = 0.5, frameon = false, loc = "lower left", handlelength = 1.5, labelspacing = 0.1, handletextpad = 0.3)
        leg1 = legend([l1, l2], ["Full", "SERTA"]; bbox_to_anchor = (0.24, 0.0, 1, 1), ncol = 1, title = "BTE", kwargs_legend...)
        leg2 = ax.legend(; frameon = false, bbox_to_anchor = (0.49, 0.0, 1, 1), ncol = 1, title = "Bubble",kwargs_legend...)
        ax.add_artist(leg1)
        leg1._legend_box.sep = 2
        leg2._legend_box.sep = 2


        ylim([1e-3, 4e3])
        yscale("log")
        xlim(extrema(nlist_SI))
        xscale("log")
        xlabel("Hole concentration (1/cm\$^2\$)"; labelpad = 0)
        ylabel("Mobility (cm\$^2\$/Vs)"; labelpad = 0)
        ax.text(0.84, 0.98, "Degen-\nerate"; ha = "left", va = "top", transform = ax.transAxes)
        ax.text(0.84, 0.98, "Non-degenerate  "; ha = "right", va = "top", transform = ax.transAxes)

        if η_tag == "smearing_5meV"
            ax.text(0.01, 0.99, "($(('a':'z')[3]))"; transform=ax.transAxes, ha="left", va="top")
        else
            ax.text(0.03, 0.97, "\$\\eta=1~\\mathrm{meV}\$"; transform=ax.transAxes, ha="left", va="top")
        end

        if η_tag == "smearing_5meV"
            do_savefig(fig, "./figures/InSe_doping_conductivity"; transparent = false)
        else
            do_savefig(fig, "./figures/InSe_doping_conductivity.$η_tag"; transparent = false)
        end
        display(fig); close(fig)
    end
end;
