include("0_common.jl")

begin
    folder_output = "$folder/../4_julia/data/wide_window"
    nk = 180
    wfpt = true

    iT = 3
    T = 100.0 * unit_to_aru(:K)
    μ = e_VBM + 0.2 * unit_to_aru(:eV)

    fig = figure(figsize=(3.4, 3.0))

    left   = 0.16
    right  = 0.97
    bottom = 0.11
    top    = 0.98

    gridspec = fig.add_gridspec(; ncols = 1, nrows = 2, hspace = 0.05, left, right, top, bottom)
    plotaxes = [fig.add_subplot(gridspec[i]) for i in 1:2]

    for (iplot, filename) in enumerate([
            "data_Dyson.smearing_5meV.doping1.wfpt$wfpt.nk$nk.jld2",
            "data_cumulant.smearing_5meV.doping1.wfpt$wfpt.nk$nk.jld2",
            "data_scFM.doping1.wfpt$wfpt.nk$nk.jld2",
        ])

        label = ["\$G_0 D_0\$", "Cumulant", "sc\$GD_0\$"][iplot]
        c = [c_FM, c_Cum, c_scFM][iplot]
        ls = ["--", "-.", "-"][iplot]
        if occursin("cumulant", filename)
            As, ωs_dense = load("$folder_output/$filename", "As_cum", "ωs_dense")
            el_i = load("$folder_output/data_scKubo_BTStates_nk$nk.jld2", "el_i")
            dos = sum(As[:, i, iT] .* el_i[i].k_weight for i in 1:el_i.n)

            i_vbm = argmax(el_i.e)
            E_vbm = ωs_dense[argmax(As[:, i_vbm, iT])]
        else
            ωs, ωs_dense, kpts, eks, Σs, occ = load("$folder_output/$filename", "ωs", "ωs_dense", "kpts", "eks", "Σs", "occ")
            dos = compute_dos(kpts, eks, ωs, Σs; elist = ωs_dense)[:, iT]
            ib, ik = argmax(eks).I
            As_vbm = @. -imag(1 / (ωs - eks[ib, ik] - Σs[:, ib, ik, iT])) / π
            E_vbm = ωs[argmax(As_vbm)]
        end
        dos .*= unit_to_aru(:eV)

        fermi = occ_fermion.(μ .- ωs_dense, T)

        zorder = 10 - iplot
        plotaxes[1].plot(_e_to_eV.(ωs_dense), dos; label, c, ls, zorder)
        plotaxes[2].plot(_e_to_eV.(ωs_dense), dos .* fermi; label, c, ls, zorder)
        for ax in plotaxes
            ax.axvline(_e_to_eV(E_vbm); c, ls = ":")
        end

        if iplot == 1
            label = "\$E_\\mathrm{VBM}^{G_0 D_0}\$"
        elseif iplot == 2
            label = "\$E_\\mathrm{VBM}^{Cumulant}\$"
        elseif iplot == 3
            label = "\$E_\\mathrm{VBM}^{scG D_0}\$"
        end
        t = plotaxes[1].text(_e_to_eV(E_vbm) - 0.01, 5e-2 * 20.0^(-iplot), label; ha = "right", c)
        t.set_bbox(Dict("facecolor" => "white", "pad" => 0., "edgecolor" =>  "white"))


        # Plot asymptotic forms
        if iplot == 1
            # Dyson
            xx = range(0.2, _e_to_eV.(ωs_dense)[end], length=101)
            plotaxes[1].plot(xx, (@. dos[end] / (xx / xx[end])^4); c = "gray", ls = "-", alpha = 0.5, lw = 2.5)

        elseif iplot == 2
            # Cumulant
            xx = range(0.3, _e_to_eV.(ωs_dense)[end], length=101)
            plotaxes[1].plot(xx, (@. dos[end] / (xx / xx[end])^4); c = "gray", ls = "-", alpha = 0.5, lw = 2.5)

        elseif iplot == 3
            # scFM
            γ0 = 35 * unit_to_aru(:meV)
            γ0_eV = γ0 / unit_to_aru(:eV)
            ω_LO_eV = ω_LO / unit_to_aru(:eV)

            xx = range(0.115, 0.3, length=101)
            ys = @. exp(-xx / (T / unit_to_aru(:eV)))
            ys .*= linear_interpolation(_e_to_eV.(ωs_dense), dos)(xx[1]) / ys[1]
            plotaxes[1].plot(xx, ys; c = "red", alpha = 0.3, lw = 2.5)
        end

    end
    plotaxes[2].axvline(_e_to_eV(μ); c = "gray", ls = "-")
    plotaxes[2].text(_e_to_eV(μ) + 0.01, 2e-5, "\$\\mu\$"; ha = "left")


    plotaxes[1].text(0.4, 5e-4, "\$\\sim 1/\\varepsilon^4\$"; c = "k", alpha = 0.6)
    plotaxes[1].text(0.24, 2e-6, "\$\\sim e^{-\\varepsilon/T}\$"; c = "r", alpha = 0.6)

    for ax in plotaxes
        ax.set_yscale("log")
        ax.set_xlim([-0.1, 0.5])
    end
    plotaxes[1].legend(; frameon = false, loc = "upper right", labelspacing=0.05, handletextpad=0.5, handlelength = 1.8, title_fontsize = fontsize, borderpad = 0.)
    plotaxes[1].set_ylim([9e-7, 1.5e1])
    plotaxes[2].set_ylim([9e-9, 4e-2])
    plotaxes[2].minorticks_off()

    plotaxes[1].set_xticklabels([])
    plotaxes[2].set_xlabel("\$\\varepsilon - \\varepsilon_\\mathrm{VBM}\$ (eV)", labelpad = 0)

    plotaxes[1].set_ylabel("DOS\$(\\varepsilon)\$ (1/eV)")
    plotaxes[2].set_ylabel("DOS\$(\\varepsilon) \\cdot f^-\\!(\\varepsilon)\$ (1/eV)")

    for (iax, ax) in enumerate(plotaxes)
        ax.text(0.02, 0.98, "($(('a':'z')[iax]))"; transform=ax.transAxes, ha="left", va="top")
    end

    fig.align_ylabels(plotaxes)
    
    do_savefig(fig, "./figures/InSe_dos"; transparent = false)
    display(fig); close(fig)
end
