include("0_common.jl")
include("../code/dev/matrix_elements.jl")

begin
    left   = 0.10
    right  = 0.98
    bottom = 0.06
    top    = 0.99

    fig = figure(figsize=(3.4, 2.8))
    gridspec = fig.add_gridspec(; nrows = 2, ncols = 2, hspace = 0.2, wspace = 0.15,
        left, right, top, bottom)
    plotaxes = [fig.add_subplot(gridspec[j, i]) for i in 1:2, j in 1:2]

    band_rng = 1:1
    band_rng_qe = 29:29

    qpts, plot_xdata = high_symmetry_kpath(model; is_2d = true, kline_density = 400)
    qpts_qe, plot_xdata_qe = high_symmetry_kpath(model; is_2d = true, kline_density = 40)

    data_qe = parse_qe_eph("../3_phonon_1d_path/ph.out", 29, 12);

    data_epw1 = parse_qe_eph("../2_epw.nopolar/epw.out", 1, 12);
    data_epw2 = parse_qe_eph("../2_epw.dipole/epw.out", 1, 12);
    data_epw3 = parse_qe_eph("../2_epw/epw.out", 1, 12);

    # Swap mode indices
    for (i1, i2, iq) in [(3, 4, 66), (3, 4, 127), (8, 9, 242), (10, 11, 21), (10, 11, 127), (10, 11, 167), (10, 11, 493)]
        data_epw1.g_abs[:, :, [i1, i2], iq:end] .= data_epw1.g_abs[:, :, [i2, i1], iq:end]
        data_epw1.e_ph[[i1, i2], iq:end] .= data_epw1.e_ph[[i2, i1], iq:end]
    end
    for (i1, i2, iq) in [(10, 11, 126), (10, 11, 167)]
        data_epw1.g_abs[:, :, [i1, i2], iq] .= NaN
    end

    for data in [data_epw2]
        for (i1, i2, iq) in [(3, 4, 66), (3, 4, 127), (8, 9, 242), (10, 11, 13), (10, 11, 127), (10, 11, 168), (10, 11, 501)]
            data.g_abs[:, :, [i1, i2], iq:end] .= data.g_abs[:, :, [i2, i1], iq:end]
            data.e_ph[[i1, i2], iq:end] .= data.e_ph[[i2, i1], iq:end]
        end
        for (i1, i2, iq) in [(10, 11, 126), (8, 9, 241)]
            data.g_abs[:, :, [i1, i2], iq] .= NaN
        end
    end

    for data in [data_epw3]
        for (i1, i2, iq) in [(3, 4, 66), (3, 4, 127), (8, 9, 242), (10, 11, 14), (10, 11, 126), (10, 11, 168), (10, 11, 500)]
            data.g_abs[:, :, [i1, i2], iq:end] .= data.g_abs[:, :, [i2, i1], iq:end]
            data.e_ph[[i1, i2], iq:end] .= data.e_ph[[i2, i1], iq:end]
        end

        for (i1, i2, iq) in [(10, 11, 126), (8, 9, 241)]
            data.g_abs[:, :, [i1, i2], iq] .= NaN
        end
    end

    for data in [data_epw1, data_epw2, data_epw3]
        # Remove Gamma acoustic phonon
        data.g_abs[1, 1, 1:3, 1] .= NaN
        data.g_abs[1, 1, 1:3, end] .= NaN
    end
    for data in [data_epw2, data_epw3]
        # Remove Gamma optical phonon
        data.g_abs[1, 1, 9:10, 1] .= NaN
        data.g_abs[1, 1, 9:10, end] .= NaN
    end

    unit_cell_mass = 353235.6850822185  # sum(model.mass[1:3:end])

    for iax in 1:2
        if iax == 1
            kwargs_qe = (; add_style = false, lw = 0, marker = "o", ms = 2.5, mew = 0.6, mec = "k", zorder = 2.5)
            # kwargs_qe = (; add_style = false, lw = 0, marker = "o", ms = 2.5, mew = 0, zorder = 2.5)
            lw = 1

            n = length(plot_xdata_qe.x)
            inds = vcat(1:2, 3:3:(n-3), (n-2):n)
        else
            kwargs_qe = (; add_style = false, lw = 0, marker = "o", ms = 2.5, mew = 0.6, mec = "k", zorder = 2.5)
            lw = 1
            inds = 1:n
            n = length(plot_xdata_qe.x)
            inds = vcat(1:5, 6:3:(n-3), (n-2):n)
        end
        plot_xdata_qe_ = (; x = plot_xdata_qe.x[inds], plot_xdata_qe.xticks, plot_xdata_qe.xlabels)

        for imode in 1:model.nmodes
        # for imode in 2:2
            ls = "-"
            c = "C$(imode-1)"
            c = "k"
            c = "w"

            y = data_qe.e_ph[imode, :]
            ElectronPhonon.plot_band_data(plotaxes[iax, 1], y[inds] ./ unit_to_aru(:meV), plot_xdata_qe_; kwargs_qe..., c)

            y = sqrt.(sum(data_qe.g_abs[band_rng_qe, band_rng_qe, imode, :].^2, dims=(1, 2))[1,1,:]) .* sqrt.(2 .* data_qe.e_ph[imode, :]) ./ unit_to_aru(:eV) .* sqrt(unit_cell_mass)
            ElectronPhonon.plot_band_data(plotaxes[iax, 2], y[inds], plot_xdata_qe_; kwargs_qe..., c)


            for (idata, data) in enumerate([data_epw1, data_epw2, data_epw3])
                c = "C$(idata-1)"
                ls = ["-", "-.", "--", ":"][idata]
                y = data.e_ph[imode, :]
                ElectronPhonon.plot_band_data(plotaxes[iax, 1], y ./ unit_to_aru(:meV), plot_xdata; add_style = false, c, ls, lw)

                y = sqrt.(sum(data.g_abs[band_rng, band_rng, imode, :].^2, dims=(1, 2))[1,1,:]) .* sqrt.(abs.(2 .* data.e_ph[imode, :])) ./ unit_to_aru(:eV) .* sqrt(unit_cell_mass)
                ElectronPhonon.plot_band_data(plotaxes[iax, 2], y, plot_xdata; add_style = false, c, ls, lw)
            end
        end
    end

    plotaxes[1, 2].legend(["DFPT", "No polar", "D", "D+Q+A"]; framealpha = 1, edgecolor = "w", loc = "upper center", labelspacing = 0.)
    # plotaxes[2].legend(["DFPT", "D+Q+A, L1", "D+Q+A, L2", "D+Q+A, L3"]; framealpha = 1)

    for ax in plotaxes
        ElectronPhonon.plot_band_data_style(ax, plot_xdata)
    end

    for iax in 1:2
        plotaxes[1, 1].set_ylabel("\$\\omega_{\\mathbf{q}\\nu}\$ (meV)"; labelpad = 0)
        plotaxes[1, 2].set_ylabel("\$D_{\\mathrm{VB},\\nu}(\\Gamma,\\mathbf{q})\$ (eV/Bohr)"; labelpad = 0)
    end
    fig.align_ylabels(plotaxes[1, :])

    plotaxes[1, 2].set_ylim(bottom = 0)

    plotaxes[2, 1].set_ylim([19.5, 25.5])
    plotaxes[2, 2].set_ylim([0, 1.3])
    plotaxes[2, 2].set_yticks([0, 1])
    plotaxes[2, 1].set_xlim([0, plot_xdata.xticks[2]])
    plotaxes[2, 2].set_xlim([0, plot_xdata.xticks[2]])


    for (iax, ax) in enumerate(plotaxes)
        pad = iax == 1 ? 1.0 : 0.0
        ax.text(0.01, 0.99, "($(('a':'z')[iax]))"; transform=ax.transAxes, ha="left", va="top", bbox = Dict("facecolor" => "w", "edgecolor" => "none", "pad" => pad), zorder = 2.5)
    end

    do_savefig(fig, "./figures/InSe_itp"; transparent = false)
    display(fig); close(fig)
end
