include("0_common.jl")

begin
    nk = 180
    wfpt = true
    ind_doping = 1
    iT = 3

    filename = "data_scFM.doping1.wfpt$wfpt.nk$nk.jld2"

    folder1 = "$folder/../4_julia/data"
    folder2 = "$folder/../4_julia/data/wide_window"

    fig = figure(figsize=(3.4, 2.8))

    left   = 0.15
    right  = 0.91
    bottom = 0.11
    top    = 0.99

    gridspec = fig.add_gridspec(; ncols = 1, nrows = 2, wspace = 0.05, hspace = 0.05, left, right, top, bottom)

    plotaxes1 = []
    plotaxes2 = []

    for iax in 1:2
        ax1 = fig.add_subplot(gridspec[iax])
        ax2 = ax1.twinx()
        sca(ax1)
        push!(plotaxes1, ax1)
        push!(plotaxes2, ax2)

        ek = -Inf

        for (ifolder, folder_) in enumerate([folder1, folder2])
            eks, ωs, Σs, kpts = load("$folder_/$filename", "eks", "ωs", "Σs", "kpts")

            if iax == 1
                ib, ik = argmax(eks).I
            else
                ib = 1
                ik = xk_to_ik(Vec3(5/60, 6/60, 0), kpts)
                ik = xk_to_ik(Vec3(0, 19/60, 0), kpts)
            end
            @info kpts.vectors[ik]
            @info _e_to_eV(eks[ib, ik])

            if ifolder == 1
                kwargs_plot = (; lw = 1, alpha = 0.5, ls = "-")
            else
                kwargs_plot = (; lw = 1, alpha = 1, ls = "--")
            end

            ek = eks[ib, ik]
            As = @. -imag(1 / (ωs - ek - Σs[:, ib, ik, iT])) / π

            ax1.plot(_e_to_eV.(ωs),   real.(Σs[:, ib, ik, iT]) ./ unit_to_aru(:eV); c = "C0", kwargs_plot...)
            ax1.plot(_e_to_eV.(ωs), .-imag.(Σs[:, ib, ik, iT]) ./ unit_to_aru(:eV); c = "C1", kwargs_plot...)
            ax2.plot(_e_to_eV.(ωs), As .* unit_to_aru(:eV); c = "k", kwargs_plot...)
        end

        ax1.axhline(0; c = "silver", lw = 1)
        if iax == 1
            ax1.plot([-0.5, -0.5], [-0.06, 0.06]; c = "silver", lw = 1, ls = "--")
            ax1.plot([-0.5, -0.5], [0.12, 0.2]; c = "silver", lw = 1, ls = "--")
        else
            ax1.axvline(-0.5; c = "silver", lw = 1, ls = "--")
        end
        ax1.set_xlim([-0.6, 0.5])
        ax1.set_xticks(-0.4:0.4:0.4)
        ax1.set_yticks(-0.04:0.04:0.12)
        if abs(_e_to_eV(ek)) < 0.01
            label = "\$n\\mathbf{k} = VBM\$"
        else
            e_text = @sprintf "%.2f" abs(_e_to_eV(ek))
            label = "\$\\mathbf{k}=(0, 19/60, 0), \$"
            label *= "\$\\varepsilon_{n\\mathbf{k}} = \\varepsilon_{VBM} - " * e_text * "\\, \\mathrm{eV}\$"
        end
        ax1.text(0.99, 0.03, label; transform = ax1.transAxes, ha = "right", va = "bottom")
    end
    # leg = legend(["\$-0.5\\,\$eV, Re", "\$-0.5\\,\$eV, Im", "\$-0.7\\,\$eV, Re", "\$-0.7\\,\$eV, Im"];
    #     title = "\$\\varepsilon_\\mathrm{min} - \\varepsilon_\\mathrm{VBM}\$", frameon = true, borderpad = 0, framealpha = 1, labelspacing = 0.3, handlelength = 1.7)
    # leg.get_frame().set_linewidth(0.0)

    ax = fig.add_axes([0, 0, 1, 1])
    ax.set_xlim([0, 1])
    ax.set_ylim([0, 1])
    ax.set_axis_off()

    # ax.add_patch(PyPlot.matplotlib.patches.Rectangle((0.17, 0.7), 0.1, 0.1))
    x1, x2 = 0.18, 0.235
    y1, y2 = 0.92, 0.865
    dy = 0.015
    for (i, c) in enumerate(["C0", "C1", "k"])
        ax.plot([x1, x2], [y1 - i * dy, y1 - i * dy]; c, lw = 1, ls = "-", alpha = 0.5)
        ax.plot([x1, x2], [y2 - i * dy, y2 - i * dy]; c, lw = 1, ls = "--", alpha = 1)
    end
    ax.text(x2 + 0.07, 0.96, "\$\\varepsilon_\\mathrm{min}\$", va = "top", ha = "left")
    ax.text(x2 + 0.01, y1 - 2dy, "\$\\varepsilon_\\mathrm{VBM} - 0.5\\,\\mathrm{eV}\$", va = "center", ha = "left")
    ax.text(x2 + 0.01, y2 - 2dy, "\$\\varepsilon_\\mathrm{VBM} - 0.7\\,\\mathrm{eV}\$", va = "center", ha = "left")


    for (ax, x1, x2, y, c) in [
        [plotaxes1[1], -0.58, -0.47, -0.04, "C0"],
        [plotaxes1[1], -0.58, -0.47,  0.03, "C1"],
        [plotaxes2[1],  0.45,  0.15, 8, "k"],
    ]
        ax.annotate(""; xy = (x1, y), xytext = (x2, y), arrowprops = Dict("arrowstyle" => "->", "color" => c))
    end
    plotaxes1[1].text(-0.45, -0.04, "Re\$\\,\\Sigma\$", va = "center", ha = "left", color = "C0")
    plotaxes1[1].text(-0.45,  0.04, "Im\$\\,\\Sigma\$", va = "center", ha = "left", color = "C1")


    plotaxes1[1].set_ylim([-0.05, 0.15])
    plotaxes2[1].set_ylim([-0.05, 0.15] .* 65)
    plotaxes1[2].set_ylim([-0.03, 0.11])
    plotaxes2[2].set_ylim([-0.03, 0.11] .* 80)
    plotaxes1[1].set_yticks(0.:0.06:0.12)
    plotaxes1[2].set_yticks(0.:0.04:0.08)
    plotaxes2[1].set_yticks(0:3:9)
    plotaxes2[2].set_yticks(0:3:6)

    plotaxes1[2].set_xlabel("\$\\varepsilon - \\varepsilon_\\mathrm{VBM}\$ (eV)"; labelpad = 0)
    plotaxes1[1].set_xticklabels([])
    plotaxes2[1].set_xticklabels([])
    for ax in plotaxes1
        ax.set_ylabel("\$\\Sigma(\\varepsilon)\$ (eV)")
    end
    for ax in plotaxes2
        ax.set_ylabel("\$A(\\varepsilon)\$ (1/eV)")
    end
    fig.align_ylabels(plotaxes1)
    fig.align_ylabels(plotaxes2)

    for (iax, ax) in enumerate(plotaxes1)
        ax.text(0.01, 0.98, "($(('a':'z')[iax]))"; ha="left", va="top", transform=ax.transAxes)
    end

    do_savefig(fig, "./figures/InSe_selfen_window"; transparent = false)
    display(fig); close(fig)
end;
