using EPSpectral
using LinearAlgebra
using StaticArrays
using Interpolations
using OhMyThreads
using PyPlot
using Base.Threads
using ChunkSplitters
using QuadGK
using Roots

# using CSV
# rgb = CSV.Tables.matrix(CSV.File(open("/home/ucl/modl/jmlihm/eph_self_consistent/linear_kbgyw_5-98_c62_n256.csv")))
# cm = PyPlot.matplotlib.colors.ListedColormap(rgb)
# PyPlot.matplotlib.colormaps.register(cm; name = "kbgyw", force=true)
# PyPlot.matplotlib.colormaps.register(cm.reversed(); name = "kbgyw_r", force=true)

# rgb = CSV.Tables.matrix(CSV.File(open("/home/ucl/modl/jmlihm/eph_self_consistent/linear_kryw_5-100_c64_n256.csv")))
# cm = PyPlot.matplotlib.colors.ListedColormap(rgb)
# PyPlot.matplotlib.colormaps.register(cm; name = "fire", force=true)
# PyPlot.matplotlib.colormaps.register(cm.reversed(); name = "fire_r", force=true)

begin
    α = 2.0
    m = 0.5
    ω₀ = 1.0
    η = 0.05
    μ = -Inf
    # μ = 0.4
    μ = 0.1
    model = FrohlichModel(α, ω₀, m, μ)

    qmax = 5.0
    # qmax = 10.0
    nq = 80
    order = 3
    qpts = polynomial_grid_3d(qmax, nq, order)

    ωs = range(-10., 30., length=301)
    k = SVector(0.2, 0., 0.)

    @time Σs_anal = get_Σ_analytic.(Ref(k), ωs .+ im * η, model)
    @time Σs_mesh = get_Σ_mesh(k, ωs .+ im * η, qpts, model; linewidth_on_electron = false);

    # Σ_mesh_shift = real(get_Σ_analytic(k, get_εk(k, model) + im * η, model) - get_Σ_mesh(k, get_εk(k, model) + im * η, qpts, model))

    # Σs_mesh .+= Σ_mesh_shift

    fig, plotaxes = subplots(1, 1, figsize=(4, 3), sharex=true)
    plotaxes.plot(ωs, real.(Σs_anal), "C0-"; label="Re Σ, analytic", alpha=0.5)
    plotaxes.plot(ωs, imag.(Σs_anal), "C1-"; label="Im Σ, analytic", alpha=0.5)
    plotaxes.plot(ωs, real.(Σs_mesh), "C0--"; label="Re Σ, mesh",)
    plotaxes.plot(ωs, imag.(Σs_mesh), "C1--"; label="Im Σ, mesh",)
    # plotaxes.plot(ωs, kramers_kronig(ωs, imag.(Σs_mesh); tail=false) .+ Σ_mesh_shift, "C2--"; label="Im Σ, mesh",)
    # plotaxes.plot(ωs, .-kramers_kronig(ωs, real.(Σs_mesh)), "C3--"; label="Im Σ, mesh",)
    axvline(get_εk(k, model), c="k", lw=1, ls="--")
    axhline(0, c="k", lw=1)
    xlim(extrema(ωs))
    # legend()
    # ylim([-1.5, 0.2])
    display(fig); close(fig)
end

begin
    α = 1.0
    m = 0.5
    ω₀ = 1.0
    occupation = 0.0
    μ = -Inf

    occupation = 0.001

    μ = (6π^2 * occupation)^(2/3) / (2m)
    μ = 0.0
    model = FrohlichModel(α, ω₀, m, μ)

    η = 0.02
    ωs = range(-6., 12., step=0.02)
    ks = range(0., 3., length=21)
    ks_dense = range(0., 3., length=51)
    ωs_dense = range(-6., 12., step=0.005)

    qmax = 4.0
    nq = 40
    order = 3
    qpts = polynomial_grid_3d(qmax, nq, order)

    S = ElectronPhononSolver(model, ωs, ks, qpts; ωs_dense, ks_dense, η, occupation)
    
    
    fig, plotaxes = subplots(4, 10, figsize=(25, 10), sharex=true, sharey="row")
    
    function _plot_solver(S)
        iax = S.iter - 3
        iax <= 0 && return
        vmax = 1.0
        plot_self_energy!(plotaxes[1, iax], S, :real; cmap="RdBu_r", vmin=-1, vmax=1, interpolation="nearest")
        plot_self_energy!(plotaxes[2, iax], S, :imag; cmap="RdBu_r", vmin=-1, vmax=1, interpolation="nearest")
        plot_spectral_function!(plotaxes[3, iax], S; cmap="viridis", norm=matplotlib.colors.LogNorm(vmin=vmax/1e3, vmax=vmax))
        plotaxes[4, iax].plot(S.ks_dense, S.spectral_sum; lw=2, c="r")
        plotaxes[4, iax].plot(S.ks_dense, S.spectral_occ; lw=2)
        plotaxes[4, iax].axhline(1, c="k", lw=1, ls="--")
        plotaxes[4, iax].set_ylim([0, 1.2])
        # display(fig)
    end
    
    compute_self_energy_analytic!(S)
    @time EPSpectral.ep_solve!(S; tol=1e-1, maxiter = 20, callback = _plot_solver, update_μ = false);
    for ax in plotaxes[1:3, :]
        # ax.set_ylim([-4, 5])
        ax.set_ylim([-4, 2])
    end
    for ax in plotaxes[4, :]
        # ax.set_ylim([0, 0.3])
    end
    display(fig); close(fig)
end;

begin
    S = ElectronPhononSolver(model, ωs, ks; ωs_dense, ks_dense, η, occupation)
    μlist = range(-2., -0.5, length=11)
    nlist = map(μlist) do μ
        S.model = FrohlichModel(α, ω₀, m, μ)
        EPSpectral.ep_solve!(S, qpts; tol=1e-2, maxiter = 30, update_μ = false);
        compute_occupation!(S)
    end
end
begin
    plot(μlist, nlist, "o-")
    yscale("log")
    fig = gcf(); display(fig); close(fig)
end

# Outer iteration to find the chemical potential
function ep_solve_given_occupation!(
    S,
    qpts,
    ;
    maxiter  :: Int64 = 100,
    tol      :: Float64 = 1e-3,
    δ        :: Float64 = 0.85,
    mem      :: Int64 = 20,
    verbose  :: Bool = true,
    update_μ :: Bool = false,
    kwargs_solver...
    )
    # Converge the solver at the given chemical potential
    ep_solve!(S, qpts; maxiter, tol, δ, mem, verbose, update_μ = false, kwargs_solver...);

    update_chemical_potential!(S)
    μ_new = find_zero(μ -> compute_occupation(S, μ) - S.occupation, extrema(S.ωs_dense))
end


begin
    α = 1.0
    m = 0.5
    ω₀ = 1.0
    occupation = 0.0
    μ = -Inf

    occupation = 0.001

    μ = (6π^2 * occupation)^(2/3) / (2m)
    model = FrohlichModel(α, ω₀, m, μ)

    η = 0.05
    ωs = range(-10., 32., step=0.05)
    ks = range(0., 4., length=21)
    ks_dense = range(0., 4., length=101)
    ωs_dense = range(-10., 32., step=0.005)

    S = ElectronPhononSolver(model, ωs, ks; ωs_dense, ks_dense, η, occupation)

    qmax = 5.0
    nq = 50
    order = 3
    qpts = polynomial_grid_3d(qmax, nq, order)

    fig, plotaxes = subplots(4, 3, figsize=(9, 10), sharex=true)
    vmax = 5.0

    for iter in 1:3
        if iter == 1
            compute_self_energy_analytic!(S)
        elseif iter == 2
            @time EPSpectral.ep_solve!(S, qpts, tol=1e-1, maxiter = 10);
        else
            @time EPSpectral.compute_self_energy_self_consistent!(S, qpts);
        end

        EPSpectral.compute_spectral_function!(S)
        nocc = EPSpectral.compute_occupation!(S)

        plot_self_energy!(plotaxes[1, iter], S, :real; cmap="RdBu_r", vmin=-1, vmax=1)
        plot_self_energy!(plotaxes[2, iter], S, :imag; cmap="RdBu_r", vmin=-1, vmax=1)
        plot_spectral_function!(plotaxes[3, iter], S; cmap="viridis", norm=matplotlib.colors.LogNorm(vmin=vmax/1e3, vmax=vmax))
        plotaxes[4, iter].plot(S.ks_dense, S.spectral_sum; lw=2, c="r")
        plotaxes[4, iter].plot(S.ks_dense, S.spectral_occ; lw=2)
        plotaxes[4, iter].axhline(1, c="k", lw=1, ls="--")
        plotaxes[4, iter].set_ylim([0, 1.2])
        # display(fig)

        # update_chemical_potential!(S)
        @info "occupation = $nocc, μ = $(S.model.μ)"
    end
    for ax in plotaxes[1:3, :]
        ax.set_ylim([-4, 5])
    end
    display(fig); close(fig)
end;

begin
    ik = 1
    fig, plotaxes = subplots(2, 1, figsize=(4, 6), sharex=true)
    sca(plotaxes[1])
    plot(S.ωs_dense, real.(S.As_dense[:, ik]), "C0-")
    # plot(S.ωs, .-imag.(S.Σs[:, ik]), "C1--"; label="Im Σ",)
    # plotaxes.plot(ωs, kramers_kronig(ωs, imag.(Σs_mesh); tail=false) .+ Σ_mesh_shift, "C2--"; label="Im Σ, mesh",)
    # plotaxes.plot(ωs, .-kramers_kronig(ωs, real.(Σs_mesh)), "C3--"; label="Im Σ, mesh",)
    # axvline(get_εk(k, model), c="k", lw=1, ls="--")
    axhline(0, c="k", lw=1)
    xlim(extrema(S.ωs_dense))
    yscale("log")
    ylim([1e-5, 1e1])
    # legend()
    # ylim([-1.5, 0.2])

    sca(plotaxes[2])
    plot(S.ωs, real.(S.Σs[:, ik]), label="Re Σ")
    plot(S.ωs, imag.(S.Σs[:, ik]), label="Im Σ")
    plot(S.ωs, S.ωs .- get_εk(S.ks[ik], S.model), "k-", label="ω - εk")
    axhline(0, c="k", lw=0.5)
    plotaxes[2].set_ylim([-5, 5])

    for ax in plotaxes
        ax.axvline(S.model.μ, c="r", lw=1)
        ax.axvline.(S.model.μ .+ [-S.model.ω₀, S.model.ω₀], c="orange", lw=1, ls="--")
        ax.axvline.(-0.25 .+ [-3, -2., -1., 0., 1.], c="k", lw=0.5, ls="--")
        ax.set_xlim([-4, 3])
    end
    plotaxes[2].legend()
    plotaxes[2].set_ylabel("A(ω)")
    plotaxes[2].set_xlabel("ω")

    fig.suptitle("k = $(S.ks[ik])")
    tight_layout()

    display(fig); close(fig)
end

function compute_phonon_self_energy(S :: ElectronPhononSolver{FrohlichModel})
    Π = zeros(ComplexF64, length(S.ωs_dense))

    Σ_itp = linear_interpolation((S.ωs, S.ks), S.Σs .- im .* S.η; extrapolation_bc = Flat());

    (; ωs_dense) = S

    dω = ωs_dense[2] - ωs_dense[1]

    for (ik, k) in enumerate(S.ks)
        if ik == 1
            k1 = 0.0
            k2 = S.ks[2] / 2
        elseif ik == length(S.ks)
            k1 = (S.ks[ik-1] + S.ks[ik]) / 2
            k2 = S.ks[ik] + (S.ks[ik] - S.ks[ik-1]) / 2
        else
            k1 = (S.ks[ik-1] + S.ks[ik]) / 2
            k2 = (S.ks[ik] + S.ks[ik+1]) / 2
        end
        weight = 4π / 3 * (k2^3 - k1^3)

        kq = k
        εk = get_εk(k, S.model)
        εkq = get_εk(kq, S.model)
        for (iω, ω) in enumerate(S.ωs_dense)
            for ν in ωs_dense
                # G1 = 1 / (ω + ν - εkq - Σ_itp(ω + ν, kq))
                # G2 = 1 / (ν - εk  - Σ_itp(ν, k))
                # if ω + ν < S.model.μ
                #     G1 = G1'
                # end
                # if ν < S.model.μ
                #     G2 = G2'
                # end

                ω + ν > S.model.μ || continue
                ν < S.model.μ || continue
                G1 =  2im * imag(1 / (ω + ν - εkq - Σ_itp(ω + ν, kq)))
                G2 = -2im * imag(1 / (ν - εk  - Σ_itp(ν, k)))

                Π[iω] += G1 * G2 * dω * weight / 2π / im
            end
        end
    end
    Π
end

begin
    # Phonon self-energy
    # @time Π = compute_phonon_self_energy(S) ./ 2
    plot(S.ωs_dense, real.(Π))
    plot(S.ωs_dense, imag.(Π))
    axvline(S.model.ω₀, c="r", lw=1)
    # xlim([-1, 2])
    axhline(0, c="k", lw=0.5)
    axvline(0, c="k", lw=0.5)
    xlim([-2, 2])
    xlim([-0.2, 0.2])
    fig = gcf(); display(fig); close(fig)
end




begin
    η_sc = 0.03

    ks_mesh = range(0., 4., length=21)

    # Σ_mesh_shift_list = map(ks_mesh) do k
    #     real(get_Σ_analytic(k, get_εk(k, model) + im * η_sc, model) - get_Σ_mesh(SVector(k, 0, 0), get_εk(k, model) + im * η_sc, qpts, model))
    # end

    @time for (ik, k) in enumerate(ks_mesh)

    end
    # # Impose Luttinger's theorem
    # k_F = sqrt(μ * 2m)
    # e_F = μ

    @time Σs_sc = get_Σ_self_consistent(k, ωs, qpts, model, Σ_itp);


    for i in 1:3
        @time for (ik, k) in enumerate(ks_mesh)
        end
        # # Impose Luttinger's theorem
        # k_F = sqrt(μ * 2m)
        # e_F = μ

    end
end


begin
    # Plot
    ks_plot = range(-4., 4., length=101)
    ωs_plot = range(-5., 9., length=201)
    η_plot = η_sc#diff(ωs_plot)[1] * 0.01

    fig, plotaxes = subplots(3, 4, figsize=(9, 7); sharex=true, sharey=true)
    vmax_ReΣ = 2.
    vmax_ImΣ = 2.
    vmax_G = 10

    fig.subplots_adjust(right=0.93)
    cbar_axes = [
        fig.add_axes([0.95, 0.68, 0.02, 0.2]),
        fig.add_axes([0.95, 0.40, 0.02, 0.2]),
        fig.add_axes([0.95, 0.12, 0.02, 0.2]),
    ]

    for (iax, Σs_data) in enumerate(Σs_sc_list)

        Σs_plot = zeros(ComplexF64, length(ωs_plot), length(ks_plot))
        As_plot = zeros(length(ωs_plot), length(ks_plot))

        Σ_output_itp = linear_interpolation((ωs, ks_mesh), Σs_data; extrapolation_bc = Flat());

        for (ik, k) in enumerate(ks_plot)
            ek = get_εk(k, model)

            @. Σs_plot[:, ik] = Σ_output_itp(ωs_plot, abs(k))
            # Σs_plot[:, ik] .+= [0.9, 0.5, 0.5, 0.5][iax]
            @. As_plot[:, ik] = -imag(1 / (ωs_plot - ek - Σs_plot[:, ik] + im * η_plot)) / π
        end

        @info extrema(real.(Σs_plot))
        @info extrema(imag.(Σs_plot))

        sca(plotaxes[1, iax])
        img = imshow(real.(Σs_plot); origin="lower", extent=[extrema(ks_plot)..., extrema(ωs_plot)...], aspect="auto", cmap="RdBu_r", vmin=-vmax_ReΣ, vmax = vmax_ReΣ)
        colorbar(img; cax = cbar_axes[1])

        sca(plotaxes[2, iax])
        img = imshow(.-imag.(Σs_plot); origin="lower", extent=[extrema(ks_plot)..., extrema(ωs_plot)...], aspect="auto", cmap="hot_r", vmin=0, vmax = vmax_ImΣ)
        colorbar(img; cax = cbar_axes[2])

        sca(plotaxes[3, iax])
        img = imshow(As_plot; origin="lower", extent=[extrema(ks_plot)..., extrema(ωs_plot)...], aspect="auto", cmap="viridis",
            # vmin=0, vmax = vmax_G,
            norm=matplotlib.colors.LogNorm(vmin=vmax_G/1e3, vmax=vmax_G),
        )
        colorbar(img; cax = cbar_axes[3])
    end

    for ax in plotaxes
        ax.axhline(model.μ, c="r", lw=1)
        ax.axhline.(model.μ .+ [-model.ω₀, model.ω₀] , c="g", lw=1, ls="--")
        ax.set_xlim(extrema(ks_plot))
        ax.set_ylim(extrema(ωs_plot))
        ax.plot(ks_plot, get_εk.(ks_plot, model), "--", c="orange", lw=1)
        # ax.set_xlim([0, 2*1.414])
        # ax.set_ylim([-3, 4] .- μ)
        ax.set_xlim([-2, 2])
        ax.set_ylim([-4, 4])
    end
    # for ax in plotaxes[3, :]
    #     ax.plot(ks_plot, get_εk.(ks_plot, model) .- α * ω₀, "--", c="black", lw=1)
    # end
    for ax in plotaxes[:, 1]
        ax.set_ylabel("ω")
    end
    for ax in plotaxes[end, :]
        ax.set_xlabel("k")
    end

    # plotaxes[1, 1].set_title("Iter 0 (Dyson)")
    # plotaxes[1, 2].set_title("Iter 1")
    # plotaxes[1, 3].set_title("Iter 2")
    # plotaxes[1, 4].set_title("Iter 3")

    for (ax, label) in zip(plotaxes[:, 1], ["Re Σ", "Im Σ", "A"])
        ax.text(-0.4, 0.5, label, rotation=90, va="center", ha="left", transform=ax.transAxes)
    end

    fig.suptitle("Frohlich model, α=$(α), μ=-Inf, m=$m, ω₀=$ω₀, μ=$μ")

    # tight_layout()

    display(fig); close(fig)
    close("all")
end


# begin
#     fig, plotaxes = subplots(3, 1, figsize=(4, 9), sharex=true)
#     sca(plotaxes[1])
#     plot(ωs, real.(Σs_anal), "k-"; label="Re Σ, analytic", alpha=0.5)
#     # plot(ωs, real.(Σs_mesh), "k--"; label="Re Σ, mesh",)
#     plotaxes[2].plot(ωs, imag.(Σs_anal), "k-"; label="Im Σ, analytic", alpha=0.5)
#     # plotaxes[2].plot(ωs, imag.(Σs_mesh), "k--"; label="Im Σ, mesh",)
#     for Σs_sc in Σs_sc_list
#         plotaxes[1].plot(ωs, real.(Σs_sc[:, 1]), "-")
#         plotaxes[2].plot(ωs, imag.(Σs_sc[:, 1]), "-")
#     end
#     axvline(get_εk(k, model), c="k", lw=1, ls="--")
#     axhline(0, c="k", lw=1)
#     xlim(extrema(ωs))
#     # legend()
#     # ylim([-1.5, 0.2])

#     sca(plotaxes[3])
#     η_plot = (ωs[2]  - ωs[1]) * 0.5
#     ik = 1
#     k = ks_mesh[ik]

#     Σs_anal_k = get_Σ_analytic.(k, ωs .+ im * η, model)

#     plot(ωs, -imag.(1 ./ (ωs .- get_εk(k, model) .- Σs_anal_k .+ im * η_plot)) ./ π, "k--")
#     # plot(ωs, -imag.(1 ./ (ωs .- get_εk(k, model) .- Σs_mesh .+ im * η_plot)) ./ π, "--")
#     for Σs_sc in Σs_sc_list
#         plot(ωs, -imag.(1 ./ (ωs .- get_εk(k, model) .- Σs_sc[:, ik] .+ im * η_plot)) ./ π)
#     end

#     for ax in plotaxes
#         ax.axvline(-α, c="k", lw=1)
#         ax.axvline(-α + ω₀, c="k", lw=1, ls="--")
#         # ax.axvline.([-ω₀, ω₀], c="k", lw=0.5, ls="--")
#         ax.set_xlim([-5, 5])
#     end
#     # plotaxes[3].set_yscale("log")
#     # plotaxes[1].set_ylim([-5.0, 0.3])
#     # plotaxes[2].set_ylim([-5.0, 0.3])
#     plotaxes[3].set_ylim([0, 1])

#     display(fig); close(fig)
# end




# function compute_spectral_function(ε, Σs, ωs_Σ, ωs_A = ωs_Σ)
#     if ωs_A == ωs_Σ
#         As = @. -imag(1 / (ωs_A - ε - Σs)) / π
#     else
#         # Linear interpolation
#         Σ_itp = linear_interpolation(ωs_Σ, Σs; extrapolation_bc = Flat());
#         As = @. -imag(1 / (ωs_A - ε - Σ_itp(ωs_A))) / π
#     end

#     return As
# end

# @views function compute_occupation(As, ωs, ks, μ, T = 1e-5)
#     @assert size(As) == (length(ωs), length(ks))

#     # Compute occupation at each k point
#     fermi = @. 1 / (exp((ωs - μ) / T) + 1)
#     dω = ωs[2] - ωs[1]
#     occ = [sum(As[:, ik] .* fermi) * dω for ik in eachindex(ks)]

#     # Linearly interpolate the occupations
#     occ_itp = linear_interpolation(ks, occ)

#     # Integrate over the Brillouin zone
#     n = quadgk(k -> 1 / 2π^2 * k^2 * occ_itp(k), extrema(ks)...)[1]

#     return n
# end
